/************************************************************************/
/*  date_change.c                                                       */
/*                                                                      */
/*  functions to calculate day_of_year to date & vice versa, leap years */
/*    etc.                                                              */
/************************************************************************/

#define TRUE    1
#define FALSE   0
#define ERROR   -1

/************************************************************************
 * date_to_day_of_year
 *
 * Description: Return the numeric day in the year for the given date
 *
 * Input parameters: day - the day (1 - 31)
 *                   mon - the month (1 - 12)
 *                   year - the year
 * Output parameters: none
 * Returns: The day of the year (1 -> 366) or -ve number on error
 *
 * Comments:
 *
 ************************************************************************/
int date_to_day_of_year (int day, int month, int year)

{

  /* check parameters */
  if (year < 0 || month < 1 || month > 12 || day < 1 ||
      day > days_in_month (month, year)) return ERROR;

  /* perform the conversion */
  for (month--; month; month--)
  {
    day = day + days_in_month (month, year);
  }
  return day;

}


/************************************************************************
 * day_of_year_to_date
 *
 * Description: Convert the day of year to a date
 *
 * Input parameters: day_of_year - the day number (1 - 366)
 *                   year - the year (0 - ??)
 * Output parameters: day - the day (1 - 31)
 *                    month - the month (1 - 12)
 * Returns: TRUE for conversion OK, FALSE otherwise
 *
 * Comments:
 *
 ************************************************************************/
int day_of_year_to_date (int day_of_year, int year, int *day, int *month)

{

  int month_days;


  /* check parameters */
  if (is_leap_year (year)) month_days = 366;
  else month_days = 365;
  if (day_of_year < 1 || day_of_year > month_days) return FALSE;

  /* perform the conversion */
  for (*month = 1; TRUE; *month += 1)
  {
    month_days = days_in_month (*month, year);
    if (day_of_year > month_days)
      day_of_year = day_of_year - month_days;
    else
    {
      *day = day_of_year;
      break;
    }
  }

  return TRUE;

}


/************************************************************************
 * days_in_month
 *
 * Description: Return the number of days in the given month
 *
 * Input parameters: month - the required month (1 - 12)
 *                   year - the year (0 - ??)
 * Output parameters: none
 * Returns: the number of days of -ve number for parameter error
 *
 * Comments:
 *
 ************************************************************************/
int days_in_month (int month, int year)

{

  /* array holding number of days / month */
  static int months [13] = {31, 0, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0};

  /* check parameters */
  if (year < 0 || month < 1 || month > 12) return ERROR;

  /* correct februarys months entry */
  if (is_leap_year (year)) months [1] = 29;
  else months [1] = 28;

  /* return the number of days */
  return (months [month -1]);

}

/************************************************************************
 * is_leap_year
 *
 * Description: Returns TRUE if the given year is a leap year
 *
 * Input parameters: year - the year to test
 * Output parameters: none
 * Returns: TRUE or FALSE or -ve number for year < 0
 *
 * Comments:
 *
 ************************************************************************/
int is_leap_year (int year)

{

  /* check parameters */
  if (year < 0) return ERROR;

  /* check for leap year */
  if (! (year % 4))
  {
    if (year % 100 == 0 && year % 400 != 0) return FALSE;
    return TRUE;
  }
  return FALSE;

}
