/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/12/94                       */
/* Program:DET_EVT                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program is processing the data coming in for seismic events.        */
/* The program looks for data in the memory pipe. If there are new buffers  */
/* in the pipe, they are copied from the pipe into two working areas. One   */
/* area is for writing data to the circle buffer and the other is for       */
/* filtering and detection.                                                 */
/* The detection algorithm may use multiple sets of parameter files, each   */
/* with different trigger criteria, different filters, different post- and  */
/* pre-event memory etc.                                                    */
/* Each buffer copied from pipe to the first working area (original data) is*/
/* written to the circle buffer, while the detection algorithm process the  */
/* filtered data.                                                           */
/* When an event occurs this program updates some queue variables and signal*/
/* to the program CAT_EVTS to take care of the archiving of the data.       */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <limits.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/kernel.h>
#include <sys/name.h>
#include <sys/types.h>
#include <sys/proxy.h>

#include "filedef.h"            /* This file defines the names of the major */
                                /* files for SEISLOG - ring etc..           */
#include "timexrou.h"
#include <math.h>
#include <time.h>
#define  MAXLEVELSTA 20
/****************************************************************************/
/*                       GLOBAL DECLARATIONS                                */
/****************************************************************************/

time_t time_of_day;
struct   date nstart;
int      fd;
smod     *ptr;
int      rng_fil_no;                   /* current ringbuffer number         */
int      sw;                           /* switch to tell which buffer       */
int      tophdr;
static float sn[MAXPOL*3+1],sd[MAXPOL*3+1];
static float save_y2[MAXSET][MAXCHA][MAXPOL*3+1];
static float save_y1[MAXSET][MAXCHA][MAXPOL*3+1];
static float save_x2[MAXSET][MAXCHA][MAXPOL*3+1];
static float save_x1[MAXSET][MAXCHA][MAXPOL*3+1];

static float save2_y2[MAXSET][MAXCHA][MAXPOL*3+1];
static float save2_y1[MAXSET][MAXCHA][MAXPOL*3+1];
static float save2_x2[MAXSET][MAXCHA][MAXPOL*3+1];
static float save2_x1[MAXSET][MAXCHA][MAXPOL*3+1];

static int nsects;

/*  Subroutine to apply an iir filter to a data sequence.                   */
/*    The filter is assumed to be stored as second order sections.          */
/*    Filtering is in-place.                                                */
/*    Zero-phase (forward and reverse) is an option.                        */

void appl(sw,par,chan)
int      sw;
int      par;
int      chan;
{
    /* System generated locals */
int      i1;
int      i2;

    /* Local variables */
int      jptr;
int      i;
int      j;
float    b0;
float    b1;
float    b2;
float    a1;
float    a2;
float    x1;
float    x2;
float    y1;
float    y2;
float    output;

/* Parameter adjustments                                                    */
/*    --data;*/

/* Function Body                                                            */
jptr = 1;
i1 = ptr->cset[par][chan];

for (j = 1; j <= i1; ++j)
{
    x1 = save_x1[par][chan][j];
    x2 = save_x2[par][chan][j];
    y1 = save_y1[par][chan][j];
    y2 = save_y2[par][chan][j];
    b0 = ptr->snc[par][chan][jptr];
    b1 = ptr->snc[par][chan][jptr + 1];
    b2 = ptr->snc[par][chan][jptr + 2];
    a1 = ptr->sdc[par][chan][jptr + 1];
    a2 = ptr->sdc[par][chan][jptr + 2];
    i2 = ptr->nsamp;
    for (i = 0; i < i2; ++i)
    {
        if(sw==0)
        {
            output = b0 * ptr->b1[i*ptr->nchan+chan] + b1 * x1 + b2 * x2;
            output -= a1 * y1 + a2 * y2;
            y2 = y1;
            y1 = output;
            x2 = x1;
            x1 = ptr->b1[i*ptr->nchan+chan];
            ptr->b1[i*ptr->nchan+chan] = output;

         }else{

            output = b0 * ptr->b2[i*ptr->nchan+chan] + b1 * x1 + b2 * x2;
            output -= a1 * y1 + a2 * y2;
            y2 = y1;
            y1 = output;
            x2 = x1;
            x1 = ptr->b2[i*ptr->nchan+chan];
            ptr->b2[i*ptr->nchan+chan] = output;

        }
    }
    jptr += 3;

    save_y2[par][chan][j]=y2;
    save_y1[par][chan][j]=y1;
    save_x2[par][chan][j]=x2;
    save_x1[par][chan][j]=x1;

}
}
/*  Subroutine to apply an iir filter to a data sequence.                   */
/*    The filter is assumed to be stored as second order sections.          */
/*    Filtering is in-place.                                                */
/*    Zero-phase (forward and reverse) is an option.                        */

void appl2(sw,par,chan)
int      sw;
int      par;
int      chan;
{
    /* System generated locals */
int      i1;
int      i2;

    /* Local variables */
int      jptr;
int      i;
int      j;
float    b0;
float    b1;
float    b2;
float    a1;
float    a2;
float    x1;
float    x2;
float    y1;
float    y2;
float    output;

/* Parameter adjustments                                                    */
/*    --data;*/

/* Function Body                                                            */
jptr = 1;
i1 = ptr->cset2[par][chan];

for (j = 1; j <= i1; ++j)
{
    x1 = save2_x1[par][chan][j];
    x2 = save2_x2[par][chan][j];
    y1 = save2_y1[par][chan][j];
    y2 = save2_y2[par][chan][j];
    b0 = ptr->snc2[par][chan][jptr];
    b1 = ptr->snc2[par][chan][jptr + 1];
    b2 = ptr->snc2[par][chan][jptr + 2];
    a1 = ptr->sdc2[par][chan][jptr + 1];
    a2 = ptr->sdc2[par][chan][jptr + 2];
    i2 = ptr->nsamp;
    for (i = 0; i < i2; ++i)
    {
        if(sw==0)
        {
            output = b0 * ptr->rsmp1[i*ptr->nchan+chan] + b1 * x1 + b2 * x2;
            output -= a1 * y1 + a2 * y2;
            y2 = y1;
            y1 = output;
            x2 = x1;
            x1 = ptr->rsmp1[i*ptr->nchan+chan];
            ptr->rsmp1[i*ptr->nchan+chan] = output;

         }else{

            output = b0 * ptr->rsmp2[i*ptr->nchan+chan] + b1 * x1 + b2 * x2;
            output -= a1 * y1 + a2 * y2;
            y2 = y1;
            y1 = output;
            x2 = x1;
            x1 = ptr->rsmp2[i*ptr->nchan+chan];
            ptr->rsmp2[i*ptr->nchan+chan] = output;

        }
    }
    jptr += 3;

    save2_y2[par][chan][j]=y2;
    save2_y1[par][chan][j]=y1;
    save2_x2[par][chan][j]=x2;
    save2_x1[par][chan][j]=x1;

}
}
/*--------------------------------------------------------------------------*/
/* Reinitialize counters and flags for detection algorithm                  */
/*--------------------------------------------------------------------------*/

void Reinit_Counter(par)
  int par;
{
int i;

ptr->ndetw[par] = 0;            /* Zero number of detection within window   */
ptr->preblk[par] =0;            /* Zero number of detections from first det.*/
for(i=0;i<ptr->nchan;i++)       /* Zero all single detection flags          */
    ptr->singdw[par][i] = 0;
}

/*--------------------------------------------------------------------------*/
/*  Voting Detector                                                         */
/*  hopefully, the duplication of some of the tests here makes it easier    */
/*  to see what is going on.                                                */
/*--------------------------------------------------------------------------*/

void Vot_Det(par)
int  par;
{
int  ks;
if (ptr->detflg[par] == 0 && ptr->ndetw[par] >= ptr->mintrg[par])
{
    ptr->detflg[par] = 1;              /* switch on the trigger             */
    ptr->ringbuffer_no[ptr->var04]=ptr->ringidx[0]; /* in what ringbuffer are we */
    ptr->qbuffile[ptr->var04]=ptr->ringidx[0];
    ptr->events_per_file[ptr->ringidx[0]]++;
    for(ks=0;ks<18;ks++)
        ptr->detnames[ptr->var04][ks]=ptr->curname[ks];
    ptr->detnames[ptr->var04][18]='\0';
    ptr->var04++;
    if(ptr->var04 >=MAXQUE)
        ptr->var04=0;

    Reinit_Counter(par);
}
else if (ptr->detflg[par] == 0 && ptr->ndetw[par] != 0)
{
    ptr->preblk[par]++;                /* increment preblk count            */
    if(ptr->preblk[par]>ptr->premax[par])
    Reinit_Counter(par);               /* give up                           */
}
else if (ptr->detflg[par] == 1 && ptr->ndet[par] <=0)
    ptr->detflg[par] = 0;              /* stop recording                    */
}
/*--------------------------------------------------------------------------*/
/* Single channel detection                                                 */
/*--------------------------------------------------------------------------*/

void Sing_Chan_Det(par, stamax, lta1, chan)
  int par;
  float stamax;
  float lta1;
  int chan;
{
int          trig_flag;                /* flag to indicate trigger          */
int          detrig_flag;              /* flag to indicate detrigger        */
float        value;                    /* value of ratio or level of signal */
unsigned int detpat1,detpat2;
/* chaflg indicates which trigger algorithm should to use to work out value:*/
/*   1 - standard, 2 - level                                                */

trig_flag    = 0;
detrig_flag  = 0;

if (ptr->chaflg[par][chan] == 1)
{
    value = stamax/lta1;
}
else if (ptr->chaflg[par][chan] == 2)
{

/*    value =(float)( fabs((double)(stamax - lta1)));*/
    value=stamax;
}
/*---           set trigger flag if the value is high enough --             */
if(value > ptr->ratio[par][chan]) 
{ 
    trig_flag = 1;
    detpat1=ptr->buff[15];
    detpat2=ptr->buff[14];
    detpat1=detpat1 | (1 << chan);
    ptr->buff[15]=detpat1;
}

/*---             work out detrigger in case it is needed                ---*/
if(value < ptr->postrt[par][chan]) detrig_flag = 1;

/*--           there are now two possibilities that matter               ---*/
/*    1) the channel was not triggered before and now it is                 */
/*    2) the channel was triggered and now it is not (ie - is detriggered)  */
/*       otherwise there should be no change to the flags                ---*/

if (trig_flag == 1 && ptr->singdt[par][chan] != 1)
{
    ptr->singdt[par][chan]=1;
    ptr->ndet[par]++;
//    printf("\nDET CH: %d %5.2f %7.2f %7.2f %s",chan,value,stamax,lta1,ptr->buff);
    if(ptr->detflg[par] == 0 && ptr->singdw[par][chan] == 0)
    {
        ptr->singdw[par][chan]=1;
        ptr->ndetw[par]++;
    }
}

else      /*-- channel has not triggered --*/
{
    if(ptr->singdt[par][chan] == 1 && detrig_flag == 1)
    {
        ptr->singdt[par][chan]=0;
        ptr->ndet[par]--;
    }
}

} /* end of sing_chan_det */

/*----------------------End: sing_chan_det---------------------------------*/

/*--------------------------------------------------------------------------*/
/* Compute STA and LTA and store max STA                                    */
/*--------------------------------------------------------------------------*/

STA_LTA(sta1,lta1,stamax,chan,lastf,par)
  float* sta1;              /* lta and sta sent as from the last data buffer*/
  float* lta1;              /* to be returned as the new values             */
  float* stamax;            /* to be returned as the max sta found          */
  int chan;                 /* number of channel to be analysed             */
  int lastf;                /* last sample to be analysed                   */
  int par;                  /* current parameter set                        */
{
register    int ix;
float       idx;                    /* Average difference in data value     */
                                    /* between 2 samples.                   */
float*      firstbufptr;
float*      nextbufptr;             /* pointer  next sample for the channel */
int         total;                  /* total number of times around the loop*/
                                    /* is number of samples between the last*/
                                    /* and first sample for that channel,   */
                                    /* divided by the no_chan               */
int no_sta = ptr->nsta[par][chan];
int no_lta = ptr->nlta[par][chan];

/* this next set of declarations is added to support  the use of the level  */
/* triggering                                                               */
static float save_vals[MAXPAR][MAXCHA][MAXLEVELSTA]={0};
/* this allows th end of the buffer to be savd for the next time the routine*/
/* is called                                                                */
float* saveptr = (float*)save_vals[par][chan];
int nsave=no_sta;             /* the number of saved values tobe used first */
static float tempbuf[BUFFER]; /* work buffer for level trigger              */
float* valptr = tempbuf;      /* pointer to work area                       */

if(sw==0)
{
    firstbufptr = ptr->b1 + chan;
}else{
    firstbufptr = ptr->b2 + chan;
}
/***nextbufptr  = firstbufptr + ptr->nchan;*/

total = (lastf- chan) / ptr->nchan;

/*----initialise max sta----*/

*stamax=0.0;

/*----analyse the samples belonging to chan for the length of the buffer--*/

for(ix=0; ix<total; ix++)
{
/*--find the difference, then slide along ready for the next two points--*/

    idx=(float)(fabs((double)(*firstbufptr)));
/***    nextbufptr += ptr->nchan;*/

/*--calculate sta and lta, and assign max if necessary--*/
    *sta1 += (idx - *sta1)/no_sta;
    *lta1 += (*sta1 - *lta1)/no_lta+0.000001;
    if(*sta1 > *stamax)
        *stamax = *sta1;
    firstbufptr+=ptr->nchan;
}
} /* end of sta_lta procedure */

/*--------------------------------------------------------------------------*/
/* Find maximum absolute amplitude channel 1                                */
/*--------------------------------------------------------------------------*/

void Max_Amp(chan, maxamp)
/*void Max_Amp(buff, chan, maxamp)*/
/*  int* buff;*/
  int chan;
  int* maxamp;
{
int i;
float*  bufptr;
if(sw==0)
{
    bufptr = ptr->b1 + chan;   /* first sample for chan */
}else{
    bufptr = ptr->b2 + chan;   /* first sample for chan */
}

*maxamp=0;
for(i=0;i<ptr->nsamp;i+=ptr->nchan)
{

    if(*bufptr > *maxamp)
        *maxamp= *bufptr;
    bufptr += ptr->nchan;
}
}

/****************************************************************************/
/*                               INIZ                                       */
/****************************************************************************/
void iniz(par, avesta, avelta)
int    par;                            /* current parameter set             */
float* avesta;                         /* average STA                       */
float* avelta;                         /* average LTA                       */

{
float  sta1;                           /* STA at current sample             */
float  lta1;                           /* LTA at current sample             */
float  stamax;                         /* Maximum STA within buffer         */
int    chan;                           /* Counter                           */
int    lastf;

*avelta=0.0;                 
*avesta=0.0;                 

lastf=ptr->nsamp * ptr->nchan;         /* samples per buffer                */

for(chan=0;chan<ptr->nchan;chan++)     /* loop through all channels         */
{
    if(ptr->chaflg[par][chan] != 0)    /* channel to be used in detection ? */
    {
        sta1=ptr->sta[par][chan];      /* set to old STA                    */
        lta1=ptr->lta[par][chan];      /* set to old LTA                    */

        STA_LTA(&sta1,&lta1,&stamax,chan,lastf,par);
        ptr->sta[par][chan] = sta1;
        ptr->lta[par][chan] = sta1;
        if(lta1 == 0.0)             /* Check for LTA=1                      */
            lta1=1.0;
    }
}
*avesta = *avesta/ptr->nchdet[par];
*avelta = *avelta/ptr->nchdet[par];
}

/****************************************************************************/
/*                               DETNET                                     */
/****************************************************************************/

void detnet(par, avesta, avelta, maxamp, dur)
  int par;                         /* current parameter set                 */
  float* avesta;
  float* avelta;
  int* maxamp;
  int dur;                         /* duration of the current trigger       */

{
float       sta1;                  /* STA at current sample                 */
float       lta1;                  /* LTA at current sample                 */

float       stamax;                /* Maximum STA within buffer             */

int         chan;                  /* Counter                               */
int         lastf;

*avelta=0.0;                       /* Average LTA for one block,all chann.  */
*avesta=0.0;                       /* Average STA for one block,all chann.  */

for(chan=0;chan<ptr->nchan;chan++)
{
               /* channel to be used in detection */
    if(ptr->chaflg[par][chan] == 1 || ptr->chaflg[par][chan] == 2)
    {
        sta1=ptr->sta[par][chan];
        lta1=ptr->lta[par][chan];

                            /* Compute STA and LTA */

        lastf = ptr ->nsamp * ptr->nchan;
        STA_LTA(&sta1,&lta1,&stamax,chan,lastf,par);
        ptr->sta[par][chan] = sta1;
        ptr->stamx[par][chan]=stamax;

                            /* Check if routine is in detection state and   */
                            /* do not update LTA if in detection. Do not    */
                            /* start updating before detection is off or    */
                            /* we have reached maximum recording time.      */
        if(!( ptr->recrd[par] == 1 && dur < ptr->maxbuf[par]))
            ptr->lta[par][chan]=lta1;     /* Update LTA                     */

        *avesta += stamax;       /* Sum max average STA all channels        */
        *avelta += lta1;         /* Sum average LTA all channels            */

        if(lta1 == 0.0)          /* Check for LTA=1                         */
            lta1=1.0;

        Sing_Chan_Det(par,stamax,lta1,chan);
                                 /* Single Channel Detection                */

    }
}
/*---all flags set by individual channels, now find out if there are enough
     to trigger --*/

        Vot_Det(par);           /****       Voting detector          *****/
*avesta = *avesta/ptr->nchdet[par];
*avelta = *avelta/ptr->nchdet[par];
}

/*---------------------------Start: Update_event---------------------------*/

Update_event(block, par, avesta, avelta, maxamp,
             par_sta, par_lta, par_maxamp, par_dur, par_fblk, sec_fblk,crc_fblk)
  int par;
  int block;            /* block number currently recording */
  float avesta, avelta;
  int maxamp;
  float *par_sta, *par_lta;
  int *par_maxamp, *par_dur, *par_fblk, *sec_fblk, *crc_fblk;

{
int      pipblk;
int      input[50];
char     syscall[100];
    int ph_ad;
    int tophdr;
    tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
    if(ptr->recrd[par] == 1)
    {
        par_dur[par]++;
        ptr->pdur[ptr->ipoint]=par_dur[par];  /* oppdatering */
        if(avesta > par_sta[par])  par_sta[par] = avesta;
        if(maxamp > par_maxamp[par]) par_maxamp[par] = maxamp;
    }
    else
    {
       ptr->recrd[par]=1;
/*----------------------disc block address ordinary event-------------------*/

       Disk_adress(block,&ph_ad);
       ph_ad=ph_ad-ptr->preblk[par] * ptr->reclength;
       if(ph_ad < tophdr)
       {
            ph_ad=ph_ad + ptr->maxblk*ptr->reclength;
       }
       par_fblk[par]=ph_ad;

       if(par_fblk[par]< 0)            /* if first trigger in event was in  */
       {                               /* previous ringbuffer file we have  */
                                       /* to wrap backwards for the correct */
                                       /* address. Physically the topheader */
                                       /* is not included here. We also have*/
                                       /* to wrap backwards the ringbuffer  */
                                       /* file index and save it for cat_evt*/
           par_fblk[par] += ptr->maxblk*ptr->reclength;

        }

/*-------------------second address in pipe ordinary event------------------*/
        sec_fblk[par]=ptr->sec_ut - ptr->preblk[par]; /* where in pipe det  */
        if(sec_fblk[par]<0)
            sec_fblk[par]+=ptr->sec_in_pip;
//printf("Diskblock %d\n",ptr->crc_in);
//printf("Start second in pipe = %d\n",sec_fblk[par]);
ptr->srt_sec[ptr->ipoint]=sec_fblk[par];
ptr->bgblk[ptr->ipoint]=sec_fblk[par]-ptr->prevnt[par]; /* pipe block number*/
                                       /* for detection start including also*/
                                       /* pre-event.                        */
                                       /* bgblk holds the start block in the*/
                                       /* pipe for the event. The time for  */
                                       /* this block is stored as well.     */
if(ptr->bgblk[ptr->ipoint] < 0)        /* take care of wrap around in pipe  */
   ptr->bgblk[ptr->ipoint]=ptr->bgblk[ptr->ipoint] + ptr->sec_in_pip;
//read_pipe_buf(sec_fblk[par],input,ptr);       /* read corresponding time-tag       */
//printf("PIPBLK=%d INPUT=%s\n",sec_fblk[par],input);
//printf("IPOINT %d\n",ptr->ipoint);
//sprintf(syscall,"/home/seislog/store %d&\n",ptr->ipoint);
//system(syscall);
/*-------------------second address in pipe P_PHASE event-------------------*/
        ptr->qpsec[ptr->pipoint] = sec_fblk[par];
        ptr->qpsmp[ptr->pipoint]=par;
        ptr->pqueue=ptr->pqueue+1;
        if(ptr->pqueue == MAXQUE)
        {
//            printf("Too many events in pqueue\n");
            ptr->pqueue=-1;
        }
        ptr->pipoint=ptr->pipoint+1;
        if(ptr->pipoint == MAXQUE)
        {
            ptr->pipoint=0;
        }

/***********  START CAT_P_PH **********************/


       par_dur[par] = ptr->preblk[par];
       ptr->pdur[ptr->ipoint]=par_dur[par];   /* oppdatering */
       par_lta[par] = avelta;

}
}

/*-------------------------------------End: Update_event ------------------*/
/*---------------------------Start: Upd_man_evt---------------------------*/

Upd_man_evt(block, par, avesta, avelta, maxamp,
             par_sta, par_lta, par_maxamp, par_dur, par_fblk, sec_fblk, crc_fblk)
  int par;
  int block;            /* block number currently recording */
  float avesta, avelta;
  int maxamp;
  float *par_sta, *par_lta;
  int *par_maxamp, *par_dur, *par_fblk, *sec_fblk, *crc_fblk;

{
    int ph_ad;
    int tophdr;
    tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
    if(ptr->recrd[MANUAL] == 1)
    {
        par_dur[MANUAL]++;
        if(avesta > par_sta[par])  par_sta[par] = avesta;
        if(maxamp > par_maxamp[par]) par_maxamp[par] = maxamp;
    }else{
        ptr->recrd[MANUAL]=1;
/*----------------------disc block address ordinary event-------------------*/

       Disk_adress(block,&ph_ad);
       ph_ad=ph_ad-ptr->preblk[MANUAL] * ptr->reclength;
       if(ph_ad < tophdr)
       {
            ph_ad=ph_ad + ptr->maxblk*ptr->reclength;
       }
       par_fblk[MANUAL]=ph_ad;

       if(par_fblk[MANUAL]< 0)            /* if first trigger in event was in  */
       {                               /* previous ringbuffer file we have  */
                                       /* to wrap backwards for the correct */
                                       /* address. Physically the topheader */
                                       /* is not included here. We also have*/
                                       /* to wrap backwards the ringbuffer  */
                                       /* file index and save it for cat_evt*/
           par_fblk[MANUAL] += ptr->maxblk*ptr->reclength;

        }

/*-------------------second address in pipe ordinary event------------------*/
/*-------------------second address in pipe P_PHASE event-------------------*/

/***********  START CAT_P_PH **********************/


       par_lta[par] = avelta;

}
}

/*-------------------------------------End: Update_event ------------------*/

/*-------------------------------------Start: Update_queue-----------------*/

Update_queue(par, par_sta, par_lta, par_maxamp, par_dur, par_fblk, sec_fblk,crc_fblk)
 int par;
 float *par_sta, *par_lta;
 int *par_maxamp, *par_dur, *par_fblk, *sec_fblk, *crc_fblk;

{
struct date start_time;
int startevt[20];
int blk;
int        hrn;                        /* hours                             */
int        min;                        /* minutes                           */
int        day;                        /* days                              */
int        mon;                        /* months                            */
int        yrn;                        /* year                              */
float      sec;                        /* seconds                           */
double     sectot;                     /* total number of seconds           */

/* set all variables for the queue */
ptr->qamp[ptr->ipoint]=par_maxamp[par];
ptr->qsta[ptr->ipoint]=par_sta[par];
ptr->qdur[ptr->ipoint]=par_dur[par];
ptr->qlta[ptr->ipoint]=par_lta[par];
ptr->qfsmp[ptr->ipoint]=par;
ptr->qfblk[ptr->ipoint]=par_fblk[par]; /* disk block number for detection   */
                                       /* including preblk. (update_event)  */
ptr->qfsec[ptr->ipoint]=sec_fblk[par]; /* pipe block number for detection   */
                                       /* including preblk. (update_event)  */
ptr->bgblk[ptr->ipoint]=sec_fblk[par]-ptr->prevnt[par]; /* pipe block number*/
                                       /* for detection start including also*/
                                       /* pre-event.                        */
                                       /* bgblk holds the start block in the*/
                                       /* pipe for the event. The time for  */
                                       /* this block is stored as well.     */
if(ptr->bgblk[ptr->ipoint] < 0)        /* take care of wrap around in pipe  */
   ptr->bgblk[ptr->ipoint]=ptr->bgblk[ptr->ipoint] + ptr->sec_in_pip;
blk=ptr->bgblk[ptr->ipoint];           /* pipe block number this event      */
read_pipe_buf_a(blk,startevt);         /* read header of this pipe block    */
Time_Conv1(startevt,&start_time,ptr);  /* extract the time for this block   */
yrn=start_time.year;
mon=start_time.month;
day=start_time.day;
hrn=start_time.hour;
min=start_time.min;
sec=start_time.secs;

//ptr->secsrt[ptr->ipoint]=sectot;       /* store total number of seconds for */
                                       /* use by cat_evts to check if the   */
                                       /* data still exists in the pipe     */

/****       ptr->qbuffile[ptr->ipoint]=ptr->ringidx[0];****/
/****printf("DET: qbuffile=%d\n",ptr->qbuffile[ptr->ipoint]);****/
     /*  printf("\nPar is: %d", par);


/* now, have to turn off the recording and complete the queue entry --*/
ptr->recrd[par]=0;

ptr->queue=ptr->queue+1;

if(ptr->queue == MAXQUE)
{
    printf("To many events in queue\n");
    ptr->queue=-1;
}
ptr->ipoint=ptr->ipoint+1;
if(ptr->ipoint == MAXQUE)
{
    ptr->ipoint=0;

}
/*if((_ev_set(det_id,7,0)) == ERROR)*/
/*    exit(_errmsg(errno,"***DET*** Unable to set event DET_ID"));*/

/* now reset the par variables which keep track of the event */
   par_dur[par] = 0;
   ptr->pdur[ptr->ipoint]=0;  /* oppdatering */
   par_sta[par] = 0;
   par_lta[par] = 0;
   par_maxamp[par] = 0;
   par_fblk[par] = 0;
   sec_fblk[par] = 0;
   crc_fblk[par] = 0;
}

/*--------------------------End: Update_queue------------------------------*/
/*--------------------------Start: Upd_man_que------------------------------*/

Upd_man_que(par, par_sta, par_lta, par_maxamp, par_dur, par_fblk, sec_fblk,crc_fblk)
 int par;
 float *par_sta, *par_lta;
 int *par_maxamp, *par_dur, *par_fblk, *sec_fblk,*crc_fblk;

{
struct date start_time;
int startevt[20];
int blk;
int        hrn;                        /* hours                             */
int        min;                        /* minutes                           */
int        day;                        /* days                              */
int        mon;                        /* months                            */
int        yrn;                        /* year                              */
float      sec;                        /* seconds                           */
double     sectot;                     /* total number of seconds           */

/* set all variables for the queue */
ptr->man_dur=par_dur[par];
//printf("MANUAL trigger in ringbuffer no. : %d\n",ptr->man_rngidx);
printf("MANUAL trigger in ringbuffer file: %s\n",ptr->man_nam);
//printf("MANUAL trigger at address number : %d\n",ptr->man_fblk);
//printf("MANUAL trigger, DURATION: %d\n",ptr->man_dur);
ptr->man_que=1;
                                       /* for detection start including also*/
                                       /* pre-event.                        */
                                       /* bgblk holds the start block in the*/
                                       /* pipe for the event. The time for  */
                                       /* this block is stored as well.     */
                                       /* number of seconds. This number is */
                                       /* the key for the start point of the*/
                                       /* event.                            */
                                       /* use by cat_evts to check if the   */
                                       /* data still exists in the pipe     */

/****       ptr->qbuffile[ptr->ipoint]=ptr->ringidx[0];****/
/****printf("DET: qbuffile=%d\n",ptr->qbuffile[ptr->ipoint]);****/
     /*  printf("\nPar is: %d", par);


/* now, have to turn off the recording and complete the queue entry --*/
ptr->recrd[par]=0;



/* now reset the par variables which keep track of the event */
   par_dur[par] = 0;
   par_sta[par] = 0;
   par_lta[par] = 0;
   par_maxamp[par] = 0;
   par_fblk[par] = 0;
   sec_fblk[par] = 0;
   crc_fblk[par] = 0;
}

/*--------------------------End: Update_man_que------------------------------*/
read_pipe_buf_a(inblk,input)
int inblk;
int* input;
{
int konst;
unsigned int* ch_ptr;
unsigned int longv;
int kp;
konst=ptr->nchan * ptr->nsamp + HEADER;
ch_ptr=ptr->pipe + inblk * konst;
for(kp=0;kp<16;kp++)
{
    longv=*(ch_ptr);
    input[kp]=longv;
    ch_ptr+=1;
}
}
/****************************************************************************/
/*                             MAIN PROGRAM                                 */
/****************************************************************************/

main()
{
FILE     *in;
pid_t    proxy;
unsigned msg;
int      i;
int      k;                            /* Counter                           */
int      par;                          /* to count parameter files          */
int      hold;                         /* Used to make sure that enough time*/
                                       /* has elapsed since start up        */
int      vent;
int      sec_no;
int      sec_prev;                     /* previous second in pipe           */
int      id;
/* the following variables keep track of the event for each parameter set   */
float    par_sta[MAXSET];
float    par_lta[MAXSET];
int      par_maxamp[MAXSET];
int      par_dur[MAXSET];
int      par_fblk[MAXSET];
int      sec_fblk[MAXSET];
int      crc_fblk[MAXSET];

float    avesta;
float    avelta;
int      maxamp;
char     mark[800];
char     syscall[100];
int      bytes;
int      bryter;
unsigned int cop;
int      det_evt_id;
int      fpl_serial_id;
int      fpl_console_id;
int      rsmp_id;
int      dec_id;

det_evt_id=qnx_name_attach(0,"det_evt");
if(det_evt_id== -1)
{
    printf("DET_EVT attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DET_EVT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DET_EVT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;

switch(ptr->unit)
{
  case 34:
  vent=60;
  break;
  case 64:
  vent=70;
  break;
  default:
  vent=15;
  break;
}
//if(ptr->unit==34)
//{
//    vent=60;
//}else{
//    vent=15;
//}


ptr->fpls_proxy=0;
ptr->fplc_proxy=0;
sw=0; 
                                 /* initially point to buffer b1      */
/*--------------------------------------------------------------------------*/
/* read next ringbuffer number into shared memory                           */
/*--------------------------------------------------------------------------*/
if(ptr->pip_dsk == 1)
    next_ring(ptr);

/*--------------------------------------------------------------------------*/
/* Initialise shared variables for queue signalling                         */
/*  and time recording                                                      */
/*--------------------------------------------------------------------------*/

ptr -> pipoint   = 0;
ptr -> pqueue    =-1;
ptr -> ipoint    = 0;               /* Set event queue pointer to 0         */
ptr -> queue     =-1;               /* Set queue pointer to -1              */
ptr -> index     = 0;               /* CHANGE                               */
ptr -> mtrig     = 0;
ptr -> ctrig     = 0;
ptr -> maxprevt  = 0;
ptr -> rng_bck   = 0;
ptr -> var04     = 0;
ptr -> man_que   = 0;
ptr->prevnt[MANUAL] = 0;
ptr->postmx[MANUAL] = 0;
ptr->maxbuf[MANUAL] = EXTBUFSIZE;
ptr->nchstr[MANUAL] = ptr->nchan;

ptr->prevnt[CONTIN] = 2;
ptr->postmx[CONTIN] = 2;
ptr->maxbuf[CONTIN] = 30;
ptr->nchstr[CONTIN] = ptr->nchan;

for(i=0;i<ptr->nchan;i++) ptr->store[CONTIN][i]=1;
for (par=0; par < MAXSET; par++){
ptr ->  preblk[par]  =0;
ptr ->  ndet[par]    =0;
ptr ->  ndetw[par]   =0;
ptr ->  postbk[par]  =0;
ptr ->  detflg[par]  =0;
ptr ->  recrd[par]   =0;

par_sta[par] = 0;
par_lta[par] = 0;
par_maxamp[par] = 0;
par_dur[par] = 0;
ptr->pdur[ptr->ipoint]=0; /* oppdatering */
par_fblk[par] = 0;
sec_fblk[par]=0;
crc_fblk[par]=0;
/*-- find out what the maximum pre-event from all the parameter sets is--*/
if (ptr->prevnt[par] > ptr->maxprevt) ptr->maxprevt = ptr->prevnt[par];
}

for (par=0; par < MAXSET; par++)
{
    for(k=0;k<ptr->nchan;k++)
    {
        ptr ->  sta[par][k]     =0;
        ptr ->  lta[par][k]     =0;
    }
}
for (par=0; par < MAXSET; par++)
{
    for(k=0;k<ptr->nchan;k++)
    {
        for(i=0;i<(MAXPOL*3+1);i++)   /* init save-vaiables for filter      */
        {
            save_x2[par][k][i]=0.0;
            save_x1[par][k][i]=0.0;
            save_y2[par][k][i]=0.0;
            save_y1[par][k][i]=0.0;
        }
    }
}
for(i=0;i<MAXCHA;i++)
    ptr->dc[i]=0.0;
/*--------------------------------------------------------------------------*/
/* For the first buffer during startup, we have to run the initialization-  */
/* routine for the algorithm selected in the parameterfile.                 */
/*--------------------------------------------------------------------------*/

ptr->sec_ut=0;

/*printf("Detec:SEC_IN_PIP=%d\n",ptr->sec_in_pip);*/

hold = ptr->sec_buffer;      /* initialise hold to indicate one buffer read */

while(ptr->kueue <= 3) sleep(1); /* any buffers in pipe to process ??       */

chain_to_proc();    /* call this once to get the time data to name the file */
                    /* this is called again to get the same data later bec. */
                    /* the pipe (kueue) indexes are not updated after thiscal*/
Time_Conv2(ptr->buff,ptr);
Start_time(ptr);
uptid(ptr);
ptr->events_per_file[ptr->ringidx[0]]=0;
//printf("RINGIDX %d\n",ptr->ringidx[0]);
//printf("PIP_DSK=%d\n",ptr->pip_dsk);
if(ptr->pip_dsk == 1)                  /* are we using hard disk rinbuffers?*/
{                                      /* yes, set switch to indicate first */
    bryter=1;                          /* ringbuffer file after startup     */
    rngopen(ptr->ringidx[0],&in,bryter);
               /* opens the next ring file and adds */
    Upd3_log(ptr);
}
/*--------------------------------------------------------------------------*/
/* Enter infinite loop                                                      */
/* The a/d-driver signals back whenever there is a buffer ready for us.     */
/* The a/d-driver always start with buffer 0 and then buffer 1.             */
/*--------------------------------------------------------------------------*/
sw=0;
ptr->var05=0;
if(ptr->actpas == 1)                   /* check for alias filtering for     */
    system("/home/seislog/dec_mat &"); /* start resampling                  */
for (;;)                               /* Infinite loop                     */
{
    for(k=0;k<ptr->maxblk;k++)
    {
        ptr->alive[2]=1;               /* set indicator to indicate active  */
/*--------------------------------------------------------------------------*/
/* Now there is new data in the data module, process it !                   */
/* The buffer that is ready, is written to the large disk circle buffer     */
/* The data is written into the circular buffer, and read into the array    */
/* buffer for processing                                                    */
/*--------------------------------------------------------------------------*/
check:
    if(ptr->kueue <= 3)                /* buffers in pipe to process ??     */
    {
        sleep(1);
        goto check;
    }
    ptr->iblk=k;
    ptr->var05++;
    if(ptr->var05 >=1000000)ptr->var05=0;
    sec_prev=ptr->sec_ut-1;            /* previous second for diff. filter  */
    if(sec_prev < 0 )                  /* check for backward wrap           */
        sec_prev=sec_prev+ptr->sec_in_pip;    /* wrap                       */
    if(ptr->actpas == 1)               /* check for alias filtering for     */
    {                                  /* resampling                        */
        chain_to_rsmp();
        for(i=0;i<ptr->nchan;i++)
            appl2(sw,0,i);
    }

for(par=0; par< ptr->active; par++)       /* Loop for every parameter set   */
{
    chain_to_proc();                      /* put buffer into buffers for:   */
                                          /* CIRCLE, FILTER etc             */
        for(i=0;i<ptr->nchan;i++)         /* Loop for every channel         */
        {
            if(ptr->chaflg[par][i] != 0)  /* check for active in detection  */
            {
                if(ptr->fltlo[par][i] != 0.0 && ptr->flthi[par][i] != 0.0)  /* Bandpass filtering ?     */
                {
                    appl(sw,par,i);       /* band pass filter               */
                }else{
                    if(ptr->filter[par][i]>0 && ptr->filter[par][i]<100)
                        flt_diff(i,sec_prev,par);
                }
            }
        }
    if(hold > ptr->maxprevt && hold >=vent) /* Check that we do not enter */
/*    if(hold > ptr->prevnt[par] && hold >=15)*/ /* Check that we do not enter */
                                             /* the detection mode before  */
                                             /* sta/lta is properly        */
                                             /* initialized after 15 second*/
                                             /* or before pre event seconds*/
                                             /* has passed                 */
    {
/*-------------------------------------------------------------------------*/
/* signal 10/11 that new buffer are ready in filter buffers                */
/*-------------------------------------------------------------------------*/
        rsmp_id=qnx_name_locate(0,"resamp",0,&cop);
        if(rsmp_id != -1)               /* FPL is there              */
        {
            ptr->buf_rdy_fpl=ptr->sec_ut;
            Trigger(ptr->resamp_proxy);
        }
        dec_id=qnx_name_locate(0,"dec_mat",0,&cop);
        if(dec_id != -1)               /* FPL is there              */
        {
            ptr->buf_rdy_fpl=ptr->sec_ut;
            Trigger(ptr->dec_mat_proxy);
        }
        fpl_serial_id=qnx_name_locate(0,"fpls",0,&cop);
        if(fpl_serial_id != -1)               /* FPL is there              */
        {
            ptr->buf_rdy_fpl=ptr->sec_ut;
            Trigger(ptr->fpls_proxy);
        }
        fpl_console_id=qnx_name_locate(0,"fplc",0,&cop);
        if(fpl_console_id != -1)               /* FPL is there             */
        {
            ptr->buf_rdy_fpl=ptr->sec_ut;
            Trigger(ptr->fplc_proxy);
        }

        detnet(par, &avesta, &avelta, &maxamp, par_dur[par]);
                                                     /* Process network    */
/*--------------------------------------------------------------------------*/
/*  if triggering, then deal with queue shared variables
    otherwise, if it doesn't trigger, but is recording, then the event has
    finished, and the queue can be updated, and CATLOG can be signalled
/*--------------------------------------------------------------------------*/
        if(ptr->detflg[par] == 1 && par_dur[par] < ptr->maxbuf[par])
        {
            Update_event(k, par, avesta, avelta,maxamp,
                    par_sta, par_lta, par_maxamp, par_dur, par_fblk,sec_fblk,crc_fblk);
        }
        else if (ptr->recrd[par] == 1)
        {
 /*-- in this case, there is no trigger, so the event should stop if it
      is recording --*/
            ptr->detflg[par] = 0;
            Update_queue(par,par_sta,par_lta,par_maxamp,par_dur,par_fblk,sec_fblk,crc_fblk);
        }
/*--------------------------------------------------------------------------*/
/* This event signals to anyone who care that one buffer has been processed */
/* The MON program waits for this event before it writes out a screen       */
/*--------------------------------------------------------------------------*/
        }else{ /* end of if statement to increment for prevnt count */
        iniz(par, &avesta, &avelta);  /* run iniz for at least 15 seconds   */

        }

    } /* end of loop for each parameter file - par control */

    sw=(sw+1)%2;                       /* switch buffer                     */
//    ptr->list[ptr->sec_ut]='r';        /* mark that buffer has been read    */
    ptr->kueue=ptr->kueue-1;           /* decrement memory queue pointer    */
    ptr->sec_ut=ptr->sec_ut+1;         /* increment memory queue pointer    */
    if(ptr->sec_ut >= ptr->sec_in_pip) /* check for wrap around             */
        ptr->sec_ut=0;

   /* increment hold if necessary */
    if (hold <= ptr->maxprevt || hold < vent)
    {
        hold += ptr->sec_buffer;
    }
    if(ptr->pip_dsk == 1)
    {
        chain_to_circle(in,k);         /* WRITE TO DISK ONLY ONCE !!        */
        last_write(ptr);



        if(k==0)                       /* first record in ringbuffer ?      */
            Upd4_log(ptr);
    }
  /* finally, update the manual trigger if it is on */

    chk_man(k, avesta, avelta,maxamp,
                         par_sta, par_lta, par_maxamp, par_dur, par_fblk,sec_fblk,crc_fblk);

} /* end of loop for each block - k control                                 */
/**** close the present ring buffer file and open the next one              */
if(ptr->pip_dsk == 1)
{
    fseek(in,0,0);                     /* position at start of file         */
    bytes=fread(mark,130,1,in);        /* read header                       */
    mark[129]='2';                     /* set indicator for finished file   */
    fseek(in,0,0);                     /* position at start of file         */
    bytes=fwrite(mark,130,1,in);       /* write back updated header         */
    fclose(in);                        /* close file                        */

//Make_link(ptr);
Do_link(ptr);

}
Upd2_log(ptr);
How_long_up(ptr);
ptr->events_per_file[ptr->ringidx[0]]=0;
//ptr->rng_idx_old[0]=ptr->ringidx[0];   /* save old index for tape backup    */
ptr->rng_bck=1;                        /* signal to backup routine to start */
bryter=0;                              /* set indicator for next file       */
if(ptr->pip_dsk == 1)                  /* pipe or disk ?                    */
{
    chain_to_proc();
    Time_Conv2(ptr->buff,ptr);
/*--------------------------------------------------------------------------*/
/* start program to write file of selected channel                          */
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/* Assign ringbuffer number, rng_fil_no number is returned                  */
/*--------------------------------------------------------------------------*/
    Update_rng_no(RNGNUMBER);
    next_ring(ptr);
    rngopen(ptr->ringidx[0],&in,bryter);               /* opens the next ring file and adds */
    Upd1_log(ptr);
}
} /* end of forever loop */
} /* end of main */
/*                     END OF MAIN PROGRAM                                  */

/*-------------------------------------------------Start: flt_diff ---------*/
/* filter specified channel with specified filter. Difference is made by    */
/* subtracting previous values from current values.                         */
/*--------------------------------------------------------------------------*/
flt_diff(ch,sec_prev,par)
int ch;
int sec_prev;
int par;
{
register short sp;
unsigned int sp1;
int indx1,indx2;
int v;
unsigned int* ch_ptr;
float* bf;
int filt_no;
int i;
float val1,val2;

filt_no=ptr->filter[par][ch];
if(sw==0)
{
    bf=ptr->b1;
}else{
    bf=ptr->b2;
}

for(i=0;i<ptr->nsamp;i++)
{
    ch_ptr=ptr->pipe+ptr->sec_ut*(ptr->nchan*ptr->nsamp+HEADER)+HEADER;
    indx1=i*ptr->nchan+ch;
    indx2=(i-filt_no)*ptr->nchan+ch;
    if(indx2 < 0)
    {
        indx2=indx2+ptr->nchan*ptr->nsamp;
        ch_ptr=ptr->pipe+sec_prev*(ptr->nchan*ptr->nsamp+HEADER)+HEADER;
    }
    val1=*(bf+indx1);
    sp1=*(ch_ptr+indx2);
    v=sp1;
    val2=(float)v;
    *(bf+indx1)=val1-val2;
}
}
/*-------------------------------------------Start: chain_to_circle---------*/
/* copy data from pipe and writes into ring buffer at the kth record
/* data is returned in buffer
/*-------------------------------------------------------------------------*/
chain_to_circle(ring,k)
    FILE* ring;
    int k;
 {
    int bytes;
    int ph_ad;
    Disk_adress(k,&ph_ad);
/*printf("Writing to: %8d\n",k*ptr->reclength+tophdr);*/
        fseek(ring,ph_ad,0);
        bytes=fwrite(ptr->buff,ptr->reclength,1,ring);
        if(bytes==0)
            clearerr(ring);
}
/*--------------------------------End: chain_to_circle----------------------*/

Disk_adress(k,ph_ad)
int* ph_ad;
{
    *ph_ad=k*ptr->reclength+tophdr;
}
/*-------------------------------------Start: chain_to_proc----------------*/
/* copy data from pipe and writes into ring buffer at the kth record
/* data is returned in buffer
/*-------------------------------------------------------------------------*/

chain_to_proc()

{
register int kp;
register int bp;
register short sp;
unsigned int longv;
unsigned int longv2;
unsigned int shortv;
float fval;
short dum;
int total;
int*   bufptr;                     /* pointer to buffer original data BUFF  */
float* bf;                         /* pointer to filter buffers B1 & B2     */
short* vaptr;                      /* pointer to buffer original data VADC  */
unsigned int* ch_ptr;            /* pointer to memory pipe                */
float sumdc;
float sumtot;
int kanal;
int sample;

/*--------------------------------------------------------------------------*/
/* transfer data from pipe to buffers                                       */
/*--------------------------------------------------------------------------*/
    total=ptr->nsamp*ptr->nchan+HEADER;

    bufptr=ptr->buff;              /* pointer to BUFF                       */
    ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER);

    for(kp=0;kp<total;kp++)        /* transfer data from pipe to BUFF       */
        *bufptr++ = *ch_ptr++;     /* including HEADER                      */

    total=ptr->nsamp*ptr->nchan;

    if(sw==0)
    {
        bf=ptr->b1;                /* pointer to B1                         */
        ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER)+HEADER;
        for(kp=0;kp<total;kp++)        /* transfer data from pipe to B1 or B2   */
        {                              /* NO HEADER                             */
            longv=*ch_ptr++;
            bp=longv;
            fval=(float)bp;
            *(bf)=fval;
/*printf("%d %f %f\n",bp,fval,ptr->b1[kp]);*/
            bf+=1;
         }

/*
         for(kanal=0; kanal < ptr->nchan; kanal++)
         {
             sumtot=0.0;
             sumdc=0.0;
             for(sample=0; sample < ptr->nsamp; sample++)
             {
                 sumdc=sumdc+ptr->b1[kanal+sample*ptr->nchan];
                 sumtot=sumtot+fabs(ptr->b1[kanal+sample*ptr->nchan]-ptr->dc[kanal]);
             }
             ptr->dc[kanal]=(ptr->dc[kanal]+sumdc/ptr->nsamp)/2;  
             ptr->to[kanal]=(ptr->to[kanal]+sumtot/ptr->nsamp)/2;
         }
*/
    }else{                         /*      or                               */
        bf=ptr->b2;                /* pointer to B2          */
        ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER)+HEADER;
        for(kp=0;kp<total;kp++)        /* transfer data from pipe to B1 or B2   */
        {                              /* NO HEADER                             */
            longv=*ch_ptr++;
            bp=longv;
            fval=(float)bp;
            *(bf)=fval;
/*printf("%d %f %f\n",bp,fval,ptr->b1[kp]);*/
            bf+=1;
         }

/*
         for(kanal=0; kanal < ptr->nchan; kanal++)
         {
             sumtot=0.0;
             sumdc=0.0;
             for(sample=0; sample < ptr->nsamp; sample++)
             {
                 sumdc=sumdc+ptr->b2[kanal+sample*ptr->nchan];
                 sumtot=sumtot+fabs(ptr->b1[kanal+sample*ptr->nchan]-ptr->dc[kanal]);
             }
             ptr->dc[kanal]=(ptr->dc[kanal]+sumdc/ptr->nsamp)/2;  
             ptr->to[kanal]=(ptr->to[kanal]+sumtot/ptr->nsamp)/2;
         }

*/


}
}

/*--------------------------------End: chain_to_proc_first-----------------*/


chain_to_rsmp()

{
register int kp;
register int bp;
register short sp;
unsigned int longv;
unsigned int longv2;
unsigned int shortv;
float fval;
short dum;
int total;
int*   bufptr;                     /* pointer to buffer original data BUFF  */
float* bf;                         /* pointer to filter buffers B1 & B2     */
short* vaptr;                      /* pointer to buffer original data VADC  */
unsigned int* ch_ptr;            /* pointer to memory pipe                */
float sumdc;
float sumtot;
int kanal;
int sample;

/*--------------------------------------------------------------------------*/
/* transfer data from pipe to buffers                                       */
/*--------------------------------------------------------------------------*/
    total=ptr->nsamp*ptr->nchan+HEADER;

    bufptr=ptr->rsmp_time;              /* pointer to BUFF                       */
    ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER);

    for(kp=0;kp<HEADER;kp++)        /* transfer data from pipe to BUFF       */
        *bufptr++ = *ch_ptr++;     /* including HEADER                      */
    total=ptr->nsamp*ptr->nchan;

    if(sw==0)
    {
        bf=ptr->rsmp1;                /* pointer to B1                         */
        ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER)+HEADER;
        for(kp=0;kp<total;kp++)        /* transfer data from pipe to B1 or B2   */
        {                              /* NO HEADER                             */
            longv=*ch_ptr++;
            bp=longv;
            fval=(float)bp;
            *(bf)=fval;
/*printf("%d %f %f\n",bp,fval,ptr->b1[kp]);*/
            bf+=1;
         }

    }else{                         /*      or                               */
        bf=ptr->rsmp2;                /* pointer to B2          */
        ch_ptr=ptr->pipe+ptr->sec_ut*((ptr->nchan*ptr->nsamp)+HEADER)+HEADER;
        for(kp=0;kp<total;kp++)        /* transfer data from pipe to B1 or B2   */
        {                              /* NO HEADER                             */
            longv=*ch_ptr++;
            bp=longv;
            fval=(float)bp;
            *(bf)=fval;
/*printf("%d %f %f\n",bp,fval,ptr->b1[kp]);*/
            bf+=1;
         }

}
}

/*-------------------Start: chk_man----------------------------*/

chk_man(k, avesta, avelta,maxamp,
                         par_sta, par_lta, par_maxamp, par_dur, par_fblk,sec_fblk,crc_fblk)
float avesta, avelta, *par_sta, *par_lta;
int maxamp, *par_maxamp, *par_dur, *par_fblk, *sec_fblk,*crc_fblk;
int k;       /* current block number */

{
int ks;
int avpa;
int diff;
if(ptr->mtrig==1)
{
//printf("%d\n",ptr->sec_ut);
ptr->qfsec[ptr->ipoint]=ptr->sec_ut; /* pipe block number for detection   */
//ptr->man_rngidx=ptr->ringidx[0];
//printf("MANUAL trigger in ringbuffer no. : %d\n",ptr->ringidx[0]);
//for(ks=0;ks<18;ks++)
//    ptr->man_nam[ks]=ptr->curname[ks];
ptr->mtrig=2;
}
     if(ptr->mtrig >= 1)  /* start or update manual */
     {
                                       /* including preblk. (update_event)  */
       Upd_man_evt(k, MANUAL, avesta, avelta, maxamp,
             par_sta, par_lta, par_maxamp, par_dur, par_fblk,sec_fblk,crc_fblk);
       ptr->man_fblk=par_fblk[MANUAL];
//       printf("MANUAL trigger at address number : %d\n",par_fblk[MANUAL]);
     }
     if( !(ptr->mtrig >= 1) && ptr->recrd[MANUAL]==1)
       Upd_man_que(MANUAL, par_sta, par_lta, par_maxamp, par_dur, par_fblk,sec_fblk,crc_fblk);
}

 /*-----------------------end: check_manual_trigger-------------------------*/

/*------------------------start: next_ring()---------------------------*/
/* looks at the directory containing the ring buffer files, and puts the    */
/* names and total number into shared memory                                */

next_ring(ptr)
smod* ptr;
{
/*--------------------------------------------------------------------------*/
/* Assign ringbuffer number, rng_fil_no number is returned                  */
/*--------------------------------------------------------------------------*/
if(Assign_rng_no(&rng_fil_no,RNGNUMBER) == ERROR)
{
    printf("DET_EVT Unable to open rng_fil_no\n");
    exit(0);
}
/* initialise the index */
ptr->ringidx[0] = rng_fil_no;
}

/****************************************************************************/
/*                 VME OS-9 & PC QNX SEISLOG Software          Version 6.0  */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Pho +47 55 213408 +47 55 213414     */ 
/*                                      Fax +47 55 233337 +47 55 320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:08/03/95                       */
/* Program:                                                                 */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Info: The following procedures are contained in this file, they are      */
/*       listed here along with the programs which use them                 */
/*                                                                          */
/* Event_Header: writes out an event header given the information           */
/*               that is needed.                                            */
/*       Used by: Detec_01.c to write headers into each ring buffer         */
/*                Cat_evts.c to write headers into event files              */
/* Event_Name:  makes up the name of an event file                          */
/*       Used by: Dsk_all.c in allocating the ring buffers for cont. record */
/*                Cat_evts.c to make up evnt file names                     */
/*                Detec_01.c to make the names of new continuous records    */ 
/****************************************************************************/

/*#include    <string.h>*/
/*#include    "shar_mem.h"*/
/*#include    <events.h>*/
/*#include    <errno.h>*/
/*#include    "filedef.h"*/           /* Include file definition for application*/
/*#include    "timexrou.h"*/
/*--------------------------------------------Start: Event_name-------------*/
/* Make file name from time-string                                          */
/*--------------------------------------------------------------------------*/
Event_Name_a(rec_time, fnam,point,par,ptr)
  struct date* rec_time;
  char* fnam;
  int point;
  int par;
  smod* ptr;
{   
int i;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */
   
   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fnam;    /* index in fnam string                           */
   
 

sprintf(cyear, "%4d", rec_time->year);
sprintf(namptr, "%2s", cyear+2);
namptr+=2;

sprintf(namptr, "%2d_", rec_time->month);
namptr+=3;
sprintf(namptr, "%2d_", rec_time->day);
namptr+=3;


sprintf(namptr, "%2d%2d_", rec_time->hour, rec_time->min);
namptr+=5;
sprintf(namptr, "%2dT.", (int) rec_time->secs);
namptr+=4;
strncpy(namptr, ptr->name, 3);                   /* station code            */
namptr+=3;
*namptr++ = '_';
sprintf(namptr, "%2d", ptr->nchstr[par]);   
namptr+=2;


switch(point)
{
    case MANUAL:
    sprintf(namptr, "_M");
    break;
    case EXTRACT:
    sprintf(namptr, "_X");
    break;
    case CONTIN:
    sprintf(namptr, "_C");
    break;
    default:
    sprintf(namptr, "_%1d", point);
    break;
}
/********************JUST FOR TESTING**********************/
/*sprintf(fnam,"9504_15_1230_30T.WNN_06_C");*/
/**********************************************************/
/* fill fnam with zeros in all blank spaces */
zero_fill_a(fnam);

}
/*------------------------------------------End: Event_Name-----------------*/

/*--------------------------------Start: zero_fill-----------------------*/

zero_fill_a(string)
  char* string;
  
{
    int i;
    for(i=0; i< strlen(string); i++) 
     if(string[i] == ' ') string[i] = '0';
     
}

/*-----------------------------------End: zero_fill----------------------*/

/*---------------------------------------Start: Event_Header----------------*/
/* Create event-header                                                      */
/*  event time is passed in the date structure rec_time, and the header
    is returned in head                                                     */
/*--------------------------------------------------------------------------*/

//30/06Event_Header_a(rec_time, nbuf, head,par,ptr,fnam)
Event_Header_a(rec_time, nbuf, head,par,ptr)
  struct date* rec_time;
  int nbuf;                       /* number of buffers                     */
  char* head;
  int par;
  smod* ptr;
//30/06  char* fnam;
{
char ring[10];
int ts;

int         start_index;            /* Index building headers               */
char* hdptr = head;                 /* text pointer to header   */  
char cyear[4];   /* help variable in cutting the century off year        */
//30/06char* namptr= fnam;    /* index in fnam string                           */
int l,j,k;                            /* counters */
int newblk;
/*printf("PAR=%d\n",par);*/
strncpy(hdptr, ptr->name, 3);       /* Put in station code    */
hdptr+=14;
//printf("DET_EVT: YEAR=%d\n",nstart.year);
sprintf(hdptr, "%4d ", nstart.year);

sprintf(cyear, "%4d", nstart.year);
//30/06sprintf(namptr, "%4s", cyear);
//30/06namptr+=4;

//30/06sprintf(namptr, "%2d_", nstart.month);
//30/06namptr+=3;
//30/06sprintf(namptr, "%2d_", nstart.day);
//30/06namptr+=3;

//30/06sprintf(namptr, "%2d%2d_", nstart.hour, nstart.min);
//30/06namptr+=5;
//30/06sprintf(namptr, "%2dT.", (int) nstart.secs);
//30/06namptr+=4;
//30/06strncpy(namptr, ptr->name, 3);                   /* station code            */
//30/06namptr+=3;
//30/06*namptr++ = '_';
//30/06sprintf(namptr, "%2d", ptr->nchstr[par]);   
//30/06namptr+=2;
//30/06sprintf(namptr, "_C");


hdptr+=5;

sprintf(hdptr,"%3d ", nstart.doy);
hdptr+=4;

sprintf(hdptr,"%2d %2d     ",nstart.month, nstart.day);
hdptr+=9;
sprintf(hdptr,"%2d %2d %6.3f",nstart.hour,nstart.min,
                              nstart.secs);

hdptr = head + 62;
if(ptr->nchstr[par] == ptr-> nchan)
{
    sprintf(hdptr, "%2d", ptr->nchan);
}else{
    sprintf(hdptr, "%2d", ptr->nchstr[par]);
}
/*printf("Here 1a\n");*/
/* Now need to put in the sample rate information */

/* SAMPLERATE start in position 64  */
hdptr = head+64;                                        
sprintf(hdptr, "%16.7f", ptr->srate);


/* find out the data type SHORT or INT */
/* NUMBER OF SAMPLES EACH CHANNEL PER EVENT + 2 OR 4 BYTE                   */
hdptr = head + 80;
if (find_data_type_a(ptr) == SHORT)
  sprintf(hdptr, "%8d       %1d",ptr->nsamp*nbuf, 2);
else
  sprintf(hdptr, "%8d       %1d",ptr->nsamp*nbuf, 4);

/* switch to indicate PC */

hdptr=head+93;
sprintf(hdptr,"%1d",2);

/* UNIT NUMBER + BYTES PER BUFFER                                           */  
hdptr = head + 96;
if(ptr->nchstr[par] == ptr-> nchan)
{
    sprintf(hdptr,"%2d      %8d",ptr->unit,ptr->reclength);
}else{
    sprintf(hdptr,"%2d      %8d",ptr->unit,ptr->newrl[par]);
}           
/*printf("Here 1b\n");*/
/* BYTES IN HEADER  + NUMBER OF SAMPLES PER CHANNEL                         */
hdptr = head + 112;
/*printf("nchstr[par] %d nchan %d\n",ptr->nchstr[par],ptr->nchan);*/
if(ptr->nchstr[par] == ptr->nchan)
{
    newblk=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
}else{
    newblk=(SIZ_EV_HDR/ptr->newrl[par]+1)*ptr->newrl[par];
}   
sprintf(hdptr,"%5d     %6d",newblk,ptr->nsamp);
/*printf("Here 1c\n");*/
k=160;
for(j=0;j<ptr->nchan;j++)
{
    if(ptr->store[par][j] == 1)
    {
        for(l=j*4;l<(j*4)+4;l++)
        {
            head[k]=ptr -> statio[l];
            head[k+4]=ptr->compon[l];
            k=k+1;
        }
    k=k+4;
    }
}
/*printf("Here 1d\n");*/
/* remove all the NULL characters that have been put in the string by the 
   sprintf statements */
   
remove_null_a( head, SIZ_EV_HDR); 
/// change ref to /home/buffer
sprintf(ptr->linknavn,"%s\n","/home/rngbuf/");
sprintf(ring,"%4d\n",ptr->ringidx[0]);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
if(ring[2]==' ')
  ring[2]='0';
if(ring[3]==' ')
  ring[3]='0';
ptr->linknavn[13]='R';
ptr->linknavn[14]=ring[0];
ptr->linknavn[15]=ring[1];
ptr->linknavn[16]=ring[2];
ptr->linknavn[17]=ring[3];
ptr->linknavn[18]='_';
sprintf(ring, "%4d ", nstart.year);
ptr->linknavn[19]=ring[0];
ptr->linknavn[20]=ring[1];
ptr->linknavn[21]=ring[2];
ptr->linknavn[22]=ring[3];
ptr->linknavn[23]='_';
sprintf(ring, "%2d", nstart.month);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[24]=ring[0];
ptr->linknavn[25]=ring[1];
ptr->linknavn[26]='_';

sprintf(ring, "%2d", nstart.day);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[27]=ring[0];
ptr->linknavn[28]=ring[1];
ptr->linknavn[29]='_';

sprintf(ring, "%2d", nstart.hour);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[30]=ring[0];
ptr->linknavn[31]=ring[1];

sprintf(ring, "%2d", nstart.min);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[32]=ring[0];
ptr->linknavn[33]=ring[1];
ptr->linknavn[34]='_';

sprintf(ring, "%2d", (int) nstart.secs);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[35]=ring[0];
ptr->linknavn[36]=ring[1];
ptr->linknavn[37]='T';
ptr->linknavn[38]='.';
ptr->linknavn[39]=ptr->name[0];
ptr->linknavn[40]=ptr->name[1];
ptr->linknavn[41]=ptr->name[2];
ptr->linknavn[42]='_';

sprintf(ring, "%2d", ptr->nchstr[par]);
if(ring[0]==' ')
  ring[0]='0';
if(ring[1]==' ')
  ring[1]='0';
ptr->linknavn[43]=ring[0];
ptr->linknavn[44]=ring[1];
ptr->linknavn[45]='_';
ptr->linknavn[46]='C';
ptr->linknavn[47]='\0';


//for(ts=0;ts<47;ts++)
//  printf("%c",ptr->linknavn[ts]);
//printf("\n");
//printf("Event_header_a: %s",ptr->linknavn);

}
/*----------------------------------------------End: Event_Header-----------*/


/*----------------------------------------------Start: Time_Conv------------*/
/* Convert and edit timestring                                              */
/*--------------------------------------------------------------------------*/
Time_Conv_a(input, event_time, ptr)
  int* input;
  struct date* event_time;
  smod* ptr;
{
char    time[15],date[15],doys[5],car;
int data_type;        /* whether to interpret the data as short or int */

sscanf((char*)input,"%s %s %s",time,date,doys);

sscanf(time,"%2d%c%2d%c%f6.3\n",&(event_time->hour),&car,
                                &(event_time->min),&car,
                                &(event_time->secs));
sscanf(date,"%2d%c%2d%c%2d\n",&(event_time->day),&car,
                              &(event_time->month),&car,
                              &(event_time->year));
sscanf(doys,"%3d\n",&(event_time->doy));
}
/*------------------------------------------------------Stop: Time_Conv-----*/
/*----------------------------------------------Start: Time_Conv1-----------*/
/* Convert and edit timestring                                              */
/*--------------------------------------------------------------------------*/
Time_Conv1(input, event_time, ptr)
  int* input;
  struct date* event_time;
  smod* ptr;
{
char    time[15],date[15],doys[5],car;
int data_type;        /* whether to interpret the data as short or int */

   sscanf((char*)input,"%s %s %s",time,date,doys);

sscanf(time,"%2d%c%2d%c%f6.3\n",&(event_time->hour),&car,
                                &(event_time->min),&car,
                                &(event_time->secs));
sscanf(date,"%2d%c%2d%c%2d\n",&(event_time->day),&car,
                              &(event_time->month),&car,
                              &(event_time->year));
sscanf(doys,"%3d\n",&(event_time->doy));
}
/*------------------------------------------------------Stop: Time_Conv1----*/


/*----------------------------------------------Start: Time_Conv------------*/
/* Convert and edit timestring                                              */
/*--------------------------------------------------------------------------*/
Time_Conv2(input, ptr)
  int* input;
  smod* ptr;
{
char    time[15],date[15],doys[5],car;
int data_type;        /* whether to interpret the data as short or int */

sscanf((char*)input,"%s %s %s",time,date,doys);

sscanf(time,"%2d%c%2d%c%f6.3\n",&(nstart.hour),&car,
                                &(nstart.min),&car,
                                &(nstart.secs));
sscanf(date,"%2d%c%2d%c%4d\n",&(nstart.day),&car,
                              &(nstart.month),&car,
                              &(nstart.year));
sscanf(doys,"%3d\n",&(nstart.doy));


}
/*------------------------------------------------------Stop: Time_Conv-----*/
/*-----------------------------Start: find_data_type()------------------*/
int find_data_type_a(ptr)
  smod *ptr;
{
     return LONG;
}
/*-----------------------------End: find_data_type()------------------*/

/*-----------------------------------Start: remove_null()-----------------*/
/* makes sure that all intermediate nulol characters are removed          */

remove_null_a(string, length)
 char* string;
 int length;
{
    int i;
    for(i=0; i<length; i++)
      if(string[i] == NULL) string[i] = ' ';
    string[length] = NULL;
    
}

/*--------------------------------------End: remove_null()----------------*/


/*------start: first_block(ptr, newblk)-------------------------------------*/

/****************************************************************************/
/* Position for first data record is computed so that the length of the     */
/* header is always a multiple of the recordlength >= 1024                  */
/****************************************************************************/
        next_block_a(ptr, newblk, blocknum, par)
         smod* ptr;
         int* newblk;
         int blocknum;
         int par;     /* current parameter set */
        
      { 
      if(ptr->nchstr[par] == ptr-> nchan)
        {
            *newblk=(SIZ_EV_HDR/ptr->reclength+blocknum)*ptr->reclength;
        }else{
            *newblk=(SIZ_EV_HDR/ptr->newrl[par]+blocknum)*ptr->newrl[par];
        }
    }


/*------------------------------------------------------Start:Reset_cpx----*/

int Reset_cpx(cpx_no,detfile)
int*  cpx_no;
char* detfile;
{
FILE  *in;                             /* pointer to contnr-file            */

if((int)(in = fopen(detfile,"r+")) == ERROR)
     return (ERROR);                   /* open contnr-file                  */
fscanf(in,"%d",cpx_no);                /* read old continous file number    */
*cpx_no = 0;                 /* update continous file number      */
rewind(in);
fprintf(in,"%8d\n",*cpx_no);           /* write new continous file number   */
fclose(in);                            /* close file                        */
return (*cpx_no);
}
/*------------------------------------------------------End:Reset_cpx------*/

/*-----------------------------Start: Write_info()----------------------*/
Write_info_a(string, dpx, dur,rec, point,ptr)
  char* string;
  int dpx;
  float dur;
  struct date* rec;
  int point;
  smod* ptr;

{
 int beamnr = 99;                         /* Dummy                        */
 int filternr= 99;                       /* Dummy                        */
 float freq  =   1.0;                        /* Dummy                        */
 float vel   =   2.0;                        /* Dummy                        */
 float azi   =   3.0;                        /* Dummy                        */
 float pwr   =   4.0;                        /* Dummy                        */ 

 float rsta = 0;
 float rlta = 0;
 float rampl = 0;
 int detsmp = 0;
 int detblk = 0;
 
if(point>-1){
 float rsta = ptr->qsta[point];
 float rlta = ptr->qlta[point];
 float rampl = ptr->qamp[point];
 int detsmp = ptr->qfsmp[point];
 int detblk = ptr->qfblk[point]/ptr->reclength;
 }

 
sprintf(string,"%16d%4d%4d%4d%4d%4d%4d%8.3f%8d%8d%16.3f\
%16.3f%16.3f%16.3f%8.3f%8.3f%8.3f%8.3f%16d%8d%8d",\
dpx,rec->year,rec->doy,rec->month,rec->day,rec->hour,
rec->min,rec->secs,beamnr,\
filternr,dur,rsta,rlta,rampl,freq,vel,azi,pwr,detblk,0,detsmp);

string[192]=32;                         /* Set NUL byte to blank after write*/
 

}

/*-----------------------------End: Write_info()----------------------*/

/*---------------------------------Start: Det_record()----------------------*/

Det_record_a(output, dpx, detfile, ptr)
char*  output;                         /* continous buffer record           */
int    dpx;                            /* continous file number             */
char*  detfile;                        /* continous files name              */
smod*  ptr;                            /* pointer  shared region            */
  
{
FILE*  dets;                           /* pointer to file                   */
int    bytes;                          /* return value after write          */
int    outblk;                         /* disk adress continous file rec    */

outblk = ((dpx-1) % ptr->nringbuf[0]) * EVTREC; /* disk adress                  */
   
/*--------------------------------------------------------------------------*/
/* Write detection record                                                   */
/*--------------------------------------------------------------------------*/

dets=fopen(detfile,"r+");
/*printf("\ndet_record: outblk is: %d", outblk);*/
fseek(dets,outblk,0);
bytes=fwrite(output,EVTREC,1,dets);
if(bytes==0)
    clearerr(dets);
fclose(dets);
}
/*-----------------------------------End: Det_record()----------------------*/

/*--------------------------------------------Start:Blank-------------------*/
/* Puts the number of blanks specified by length in the string argument     */
Blank_a(string, length)
  char *string;
  int length;
{
    int i;
    for(i=0;i<length;i++){
        string[i] = 32;
    }
    string[length] = 0;
}
/*--------------------------------------------Stop:Blank--------------------*/
/*--------------------------------------------------------------------------*/
/* called by: BUF_PIPE                                                      */
int Assign_rng_no(rng_fil_no,rngfile)
   int* rng_fil_no;
   char* rngfile;
{
int newnr;
FILE        *in;                  /* Pointer to detnr-file                  */

in = fopen(rngfile,"r+");
if(in == 0)
{
    sprintf(ptr->err_msg,"DET_EVT: Unable to open file with ringbuffer number");
    errlog(ptr);
    exit(0);
}
fscanf(in,"%d",&newnr);                /* Read current ring-file number     */
*rng_fil_no=newnr;
fclose(in);                            /* Close file                        */
return (*rng_fil_no);
}
Update_rng_no(rngfile)
   char* rngfile;
{
int newnr;
FILE        *in;                  /* Pointer to detnr-file                  */

in = fopen(rngfile,"r+");
if(in == 0)
{
    sprintf(ptr->err_msg,"DET_EVT: Unable to open file with ringbuffer number");
    errlog(ptr);
    exit(0);
}
fscanf(in,"%d",&newnr);                     /* Read old detection number    */
newnr=newnr+1;
if(newnr >= ptr->nringbuf[0])     /* wrap                             */
    newnr=0;
rewind(in);
fprintf(in,"%10d",newnr);
fclose(in);                            /* Close file                        */
}
/*-----------start: rngopen(ptr)--------------------------------------------*/
/* opens the next ring buffer file for writing and puts a header on at the  */
/* beginning so that the files have the same format as the files in the     */
/* events directory                                                         */
/* the file is left positioned for the start of writing                     */
/*--------------------------------------------------------------------------*/

rngopen(nxt_rbf,in,bryter)
int nxt_rbf;
FILE* *in;
int bryter;
{ 
FILE *idx;
char lnknam[100];
char head[SIZ_EV_HDR+1];
struct date start;
char idx_log[400];
char fname[200];
char syscall[100];
char smark[950];
int bytes;
int newblk;
int cpx;
char rngbuf[50];
int ip;
int i;
char indexrec[10];
char buf[50];
int lr;
FILE *lnk;
//03/07/00char cpxrec[EVTREC+1];
static char path[]="/home/buffer/RXXXXl";
float dur = ptr->maxblk;

for(lr=0;lr<20;lr++)
    rngbuf[lr]=' ';
sprintf(indexrec,"%4d\n",nxt_rbf);
if(indexrec[0]==' ')
    indexrec[0]='0';
if(indexrec[1]==' ')
    indexrec[1]='0';
if(indexrec[2]==' ')
    indexrec[2]='0';
if(indexrec[3]==' ')
    indexrec[3]='0';
path[14]=indexrec[0];
path[15]=indexrec[1];
path[16]=indexrec[2];
path[17]=indexrec[3];
for(lr=0;lr<18;lr++)
{
    rngbuf[lr]=path[lr];
    ptr->curname[lr]=path[lr];
}
rngbuf[18]='\0';
ptr->curname[18]='\0';
/* terje */
//printf("RNGOPEN: %s\n",rngbuf);
/*--------------------------------------------------------------------------*/
/* open the corresponding ringbuffer file                                   */
/*--------------------------------------------------------------------------*/
if((*in=fopen(rngbuf, "r+"))==NULL)
{
    sprintf(ptr->err_msg,"DET_EVT: Uable to open %s",rngbuf);
    errlog(ptr);
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* extract linkname from header                                             */
/*--------------------------------------------------------------------------*/
fseek(*in,0,0);                        /* position at start of file         */
bytes=fread(smark,950,1,*in);          /* read header                       */
fseek(*in,0,0);
/*--------------------------------------------------------------------------*/
/* assign a continous sequence number to each ringbuffer file               */
/*--------------------------------------------------------------------------*/
Assign_cpx(&cpx, CONNUMBER);
/* now get the header information */
Blank_a(head, SIZ_EV_HDR);
//03/07/00Blank_a(cpxrec, EVTREC);
//30/06Event_Header_a(&start,ptr->maxblk,head,CONTIN,ptr,lnknam);
Event_Header_a(&start,ptr->maxblk,head,CONTIN,ptr);
if(bryter == 1)                        /* first after startup ?             */
{
    head[128]='1';                     /* set indicator for first file      */
    head[129]='1';                     /* set indicator for current file    */
}else{
    head[128]='0';                     /* set indicator for next file       */
    head[129]='1';                     /* set indicator for current file    */
}           
/*--------------------------------------------------------------------------*/
/* add into header the detection record                                     */
/*--------------------------------------------------------------------------*/
Write_info_a(head+688, cpx, dur, &start, -1, ptr);   
//03/07/00Write_info_a(cpxrec, cpx, dur, &start, -1, ptr);
strncpy(head+688+208, rngbuf+13, 5);


remove_null_a(head, SIZ_EV_HDR);
//03/07/00remove_null_a(cpxrec,  EVTREC);
/*--------------------------------------------------------------------------*/
/* Write event header                                                       */
/*--------------------------------------------------------------------------*/
fseek(*in,0,0);
bytes=fwrite(head,SIZ_EV_HDR,1,*in);    
if(bytes==0)
    clearerr(*in);
        
/* now position the file to start writing */
next_block_a(ptr,&newblk,1,CONTIN);
/*    printf("\n %d\n", newblk);*/
fseek(*in,newblk,0);   
}

Upd1_log(ptr)
smod* ptr;
{
char idx_log[400];
char rngnam[100];
char start_time[30];
char status[20];
char buf[50];
int  i;
int  k;
FILE *idx;
idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
    time_of_day=time(NULL);
    sprintf(start_time,"%s\n",_ctime(&time_of_day,buf));
    for(k=0;k<24;k++)
        idx_log[k+55]=start_time[k];
    for(k=0;k<4;k++)                   /* move in 4 digit YEAR              */
        idx_log[k+19]=idx_log[k+75];
    sprintf(status,"%2d",nstart.month);
    if(status[0]==' ') status[0]='0';  /* move in MONTH                     */
    idx_log[24]=status[0];
    idx_log[25]=status[1];
    sprintf(status,"%2d",nstart.day);
    if(status[0]==' ') status[0]='0';  /* move in DAY                       */
    idx_log[27]=status[0];
    idx_log[28]=status[1];
    sprintf(status,"%2d",nstart.hour);
    if(status[0]==' ') status[0]='0';  /* move in HOUR                      */
    idx_log[30]=status[0];
    idx_log[31]=status[1];
    sprintf(status,"%2d",nstart.min);
    if(status[0]==' ') status[0]='0';  /* move in MINUTE                    */
    idx_log[32]=status[0];
    idx_log[33]=status[1];
    sprintf(status,"%2d",(int)nstart.secs);
    if(status[0]==' ') status[0]='0';  /* move in SECONDS                   */
    idx_log[35]=status[0];
    idx_log[36]=status[1];
    sprintf(status,"1 REC");      /* EMPTY FULL NOT-FULL */
    for(k=0;k<5;k++)
        idx_log[k+87]=status[k];

if(idx_log[167]=='3')
    for(k=0;k<5;k++)
        idx_log[k+167]=' ';

    rewind(idx);
    fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
    fwrite(idx_log,320,1,idx);
    fclose(idx);
}
Make_link(ptr)
smod* ptr;
{
int      sk;
int      len;
char     ringnavn[50],linknavn[50];
char idx_log[400];
char cmd[100];
char cmd1[PATH_MAX+1];
char cmd2[100];
char cmd3[100];
FILE *idx;
idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
// change ref to /home/buffer
sprintf(linknavn,"%s\n","/home/buffer/");
//("%s\n",linknavn);
for(sk=0;sk<35;sk++)
    linknavn[sk+13]=idx_log[sk+13];
linknavn[18]='_';
linknavn[sk+13]='\0';
//printf("Make_link: %s\n",linknavn);

for(sk=0;sk<18;sk++)
    ringnavn[sk]=idx_log[sk];
ringnavn[sk]='\0';

//("%s\n",ringnavn);
    fclose(idx);
for(sk=0;sk<18;sk++)
    cmd3[sk]=linknavn[sk];
cmd3[sk]='\0';

sprintf(cmd2,"/home/seislog/cln_lnk %s\n",cmd3);
system(cmd2);

sprintf(cmd,"ln -s %s %s\n",ringnavn,linknavn);
//("%s\n",cmd);
system(cmd);
////sprintf(ptr->err_msg,"DET_EVT: %5d finished %s",ptr->ringidx[0],ringnavn);
////errlog(ptr);
////sprintf(ptr->err_msg,"lnk to: %s",linknavn);
////errlog(ptr);
}
Upd2_log(ptr)
smod* ptr;
{
char idx_log[400];
char rngnam[100];
char stopp_time[30];
char buf[50];
int  i;
int  k;
FILE *idx;
idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
    time_of_day=time(NULL);
    sprintf(stopp_time,"%s\n",_ctime(&time_of_day,buf));
    for(k=0;k<24;k++)
        idx_log[k+135]=stopp_time[k];
    rewind(idx);
    fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
    fwrite(idx_log,320,1,idx);
    fclose(idx);
}
Upd3_log(ptr)
smod* ptr;
{
char idx_log[400];
char rngnam[100];
char start_time[30];
char status[20];
char buf[50];
int  i;
int  k;
FILE *idx;
idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
    time_of_day=time(NULL);
    sprintf(start_time,"%s\n",_ctime(&time_of_day,buf));
    for(k=0;k<24;k++)
        idx_log[k+55]=start_time[k];
    for(k=0;k<4;k++)                   /* move in 4 digit YEAR              */
        idx_log[k+19]=idx_log[k+75];
    sprintf(status,"%2d",nstart.month);
    if(status[0]==' ') status[0]='0';  /* move in MONTH                     */
    idx_log[24]=status[0];
    idx_log[25]=status[1];
    sprintf(status,"%2d",nstart.day);
    if(status[0]==' ') status[0]='0';  /* move in DAY                       */
    idx_log[27]=status[0];
    idx_log[28]=status[1];
    sprintf(status,"%2d",nstart.hour);
    if(status[0]==' ') status[0]='0';  /* move in HOUR                      */
    idx_log[30]=status[0];
    idx_log[31]=status[1];
    sprintf(status,"%2d",nstart.min);
    if(status[0]==' ') status[0]='0';  /* move in MINUTE                    */
    idx_log[32]=status[0];
    idx_log[33]=status[1];
    sprintf(status,"%2d",(int)nstart.secs);
    if(status[0]==' ') status[0]='0';  /* move in SECONDS                   */
    idx_log[35]=status[0];
    idx_log[36]=status[1];
    sprintf(status,"1 REC");      /* EMPTY FULL NOT-FULL */
    for(k=0;k<5;k++)
        idx_log[k+87]=status[k];
    sprintf(status,"3 RST");      /* EMPTY FULL NOT-FULL RESTART            */
    for(k=0;k<5;k++)
    {
        idx_log[k+167]=status[k];
    }
    rewind(idx);
    fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
    fwrite(idx_log,320,1,idx);
    fclose(idx);
}
Upd4_log(ptr)
smod* ptr;
{
char idx_log[400];
char rngnam[100];
char stopp_time[30];
char buf[80];
int  i;
int  k;
FILE *idx;
idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
    sprintf(buf,"%s\n",ptr->buff);     /* insert clock status from databuf  */
    for(k=0;k<3;k++)
        idx_log[k+115]=buf[k+28];
    idx_log[96]='0';                   /* zero up time when new ringbuffer  */
    idx_log[97]='0';
    idx_log[98]='0';
    idx_log[99]=' ';
    idx_log[100]='0';
    idx_log[101]='0';
    idx_log[102]=' ';
    idx_log[103]='0';
    idx_log[104]='0';
    rewind(idx);
    fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
    fwrite(idx_log,320,1,idx);
    fclose(idx);
}
How_long_up(ptr)
smod* ptr;
{
char idx_log[400];
char rngnam[100];
char up_time[30];
char buf[50];
int  i;
int  k;
FILE *idx;
int cpxn;
int minu;
char totevents[10];
char status[10];
int totday;
int tothour;
int totmin;
float mi;
float dager;
int hel;
float timer;
float ti;
int rest;
FILE  *in;                             /* pointer to contnr-file            */
if((int)(in = fopen("/home/seislog/pr/contnr","r+")) == ERROR)
{
    printf("Cant open file:CONTNR\n");
    exit(0);
}
fscanf(in,"%d",&cpxn);                 /* read old continous file number    */
minu=cpxn * ptr->nmin_per_ring[0];
fclose(in);                            /* close file                        */
mi=(float)minu;
dager=mi/1440.;
totday=(int)dager;
rest=minu-totday * 1440;
ti=(float)rest;
timer=ti/60;
tothour=(int)timer;
totmin=minu-totday*1440-tothour*60;

idx=fopen("/home/buffer/idx_log","r+");
fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
fread(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
    sprintf(status,"2 FIN\n");
    idx_log[87]=status[0];
    idx_log[88]=status[1];
    idx_log[89]=status[2];
    idx_log[90]=status[3];
    idx_log[91]=status[4];
    sprintf(up_time,"%3d %2d:%2d\n",totday,tothour,totmin);
    if(up_time[0]==' ')up_time[0]='0';
    if(up_time[1]==' ')up_time[1]='0';
    if(up_time[2]==' ')up_time[2]='0';

    if(up_time[4]==' ')up_time[4]='0';
    if(up_time[5]==' ')up_time[5]='0';

    if(up_time[7]==' ')up_time[7]='0';
    if(up_time[8]==' ')up_time[8]='0';
    for(k=0;k<9;k++)
        idx_log[k+96]=up_time[k];
    sprintf(totevents,"%5d",ptr->events_per_file[ptr->ringidx[0]]);
    if(totevents[0]==' ')totevents[0]='0';
    if(totevents[1]==' ')totevents[1]='0';
    if(totevents[2]==' ')totevents[2]='0';
    if(totevents[3]==' ')totevents[3]='0';
    if(totevents[4]==' ')totevents[4]='0';
    for(k=0;k<5;k++)
        idx_log[k+109]=totevents[k];
    rewind(idx);
    fseek(idx,ptr->ringidx[0]*320,SEEK_SET);
    fwrite(idx_log,320,1,idx);
    fclose(idx);
    if((int)(in = fopen("/home/buffer/uptime","w")) == ERROR)
    {
        printf("Cant open file: UPTIME\n");
        exit(0);
    }
    rewind(in);
    fwrite(up_time,9,1,in);
    fclose(in);
}
Start_time(ptr)
smod* ptr;
{
char starttid[30];
int  k;
char buf[50];
FILE *idx;
idx=fopen("/home/buffer/start_time","w");
rewind(idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
for(k=0;k<30;k++)
    starttid[k]=' ';
time_of_day=time(NULL);
sprintf(starttid,"%s",_ctime(&time_of_day,buf));
fwrite(starttid,24,1,idx);
fclose(idx);
}
/*------------------------------------------------------Start:Assign_cpx----*/

int Assign_cpx(cpx_no,detfile)
int*  cpx_no;
char* detfile;
{
FILE  *in;                             /* pointer to contnr-file            */
if((int)(in = fopen(detfile,"r+")) == ERROR)
{
    printf("Cant open CONTNR\n");
     return (ERROR);                   /* open contnr-file                  */
}
fscanf(in,"%d",cpx_no);                /* read old continous file number    */
*cpx_no = *cpx_no + 1;                 /* update continous file number      */
rewind(in);
fprintf(in,"%8d\n",*cpx_no);           /* write new continous file number   */
fclose(in);                            /* close file                        */
return (*cpx_no);
}
/*------------------------------------------------------End:Assign_cpx------*/
uptid(ptr)
smod* ptr;
{
char up_time[30];
FILE  *in;                             /* pointer to contnr-file            */
if((int)(in = fopen("/home/buffer/uptime","w")) == ERROR)
{
    printf("Cant open file: UPTIME\n");
    exit(0);
}
rewind(in);
sprintf(up_time,"000 00:00");
    fwrite(up_time,9,1,in);
    fclose(in);
}
last_write(ptr)
smod* ptr;
{
char starttid[30];
int  k;
char buf[50];
FILE *idx;
idx=fopen("/home/buffer/last_write","w");
rewind(idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
for(k=0;k<30;k++)
    starttid[k]=' ';
time_of_day=time(NULL);
sprintf(starttid,"%s",_ctime(&time_of_day,buf));
sprintf(buf,"%4d",ptr->ringidx[0]);
starttid[24]=buf[0];
starttid[25]=buf[1];
starttid[26]=buf[2];
starttid[27]=buf[3];
fwrite(starttid,28,1,idx);
fclose(idx);
}
Do_link(ptr)
smod* ptr;
{
int      sk;
int      len;
char     ringnavn[50],linknavn[50];
char idx_log[400];
char cmd[100];
char cmd1[PATH_MAX+1];
char cmd2[100];
char cmd3[100];

//printf("Make_link: %s\n",ptr->linknavn);

for(sk=0;sk<18;sk++)
    ringnavn[sk]=ptr->linknavn[sk];
ringnavn[sk]='\0';
ringnavn[6]='b';
ringnavn[7]='u';
ringnavn[8]='f';
ringnavn[9]='f';
ringnavn[10]='e';
ringnavn[11]='r';
//("%s\n",ringnavn);
for(sk=0;sk<18;sk++)
    cmd3[sk]=ptr->linknavn[sk];
cmd3[sk]='\0';

sprintf(cmd2,"/home/seislog/cln_lnk %s\n",cmd3);
system(cmd2);
//printf("ln -s %s %s\n",ringnavn,ptr->linknavn);
sprintf(cmd,"ln -s %s %s\n",ringnavn,ptr->linknavn);
//("%s\n",cmd);
system(cmd);
////sprintf(ptr->err_msg,"DET_EVT: %5d finished %s",ptr->ringidx[0],ringnavn);
////errlog(ptr);
////sprintf(ptr->err_msg,"lnk to: %s",ptr->linknavn);
////errlog(ptr);
}
