/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      Author: David Scott                 */
/*                                              BGS, Murchison House,       */
/*                                              Edinburgh                   */
/*                                                                          */
/*                                      Date:12/09/97                       */
/* Program: ILI_SEIS.C                                                      */
/*                                                                          */
/* Last update:   Added workarounds for two known error conditions:         */
/*     1.  Phantom packets produced by the ILI continuing to write into     */
/*         the buffer after we have told it to stop, and thus kicking the   */
/*         proxy so that the driver loop is executed too often.  A phantom  */
/*         packet always seems to have packet no = 0, so if we detect this, */
/*         skip the rest of that iteration.                                 */
/*     2.  Some packets contain an out of date timestring even though they  */
/*         are marked as valid packets by the ILI.  Compare the time with   */
/*         last second's, and if it is the same, increment it.              */
/*                                       DJS 03/02/98                       */
/*                                                                          */
/* Program description: Driver code for 24-bit ili support                  */
/*     Requires interrupt routines in int.c
/*                                                                          */
/****************************************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>
#include <i86.h>
#include <sys/irqinfo.h>
#include <sys/proxy.h>
#include "shar_mem.h"
#include <errno.h>
#include <fcntl.h>
#include <sys/mman.h>

#include <math.h>         /* for synthetic data only */

#include "ili_24b.h"

#define TRUE	1
#define FALSE	0

/****************************************************************************/
/*    Function prototypes                                                   */
/****************************************************************************/

int start_ili_24b (void);
void stop_ili_24b (void);
int sync (int *);
int check_channel_errors (void);
void convert2 (void);
unsigned short int convert_data (union Join*, int);
void reorder_channels (void);
void extract_timestring (void);
void increment_last_time (void);

extern int date_to_day_of_year (int, int, int);
extern int day_of_year_to_date (int, int, int *, int *);
extern int days_in_month (int, int);
extern int is_leap_year (int);

/****************************************************************************/
/*    Global declarations                                                   */
/****************************************************************************/

smod *ptr;                               /* pointer to shared memory region */
static int irq_handler_id = -1;                /* ILI interrupt handler id  */
static int timer_irq_handler_id = -1;         /* timer interrupt handler id */
volatile char buffer[2][MAX_TRANSFER_SIZE];                 /* data buffers */
int ilichan[MAX_ILI_CHAN];   /* mapping of ILI channels to SEISLOG channels */
int sample_order[MAX_ILI_CHAN]; /* ilichan sorted in order samples received */
int transform_order[16][3];             /* 2D array used to re-sort ilichan */
volatile int current_buffer = 0;                        /* buffer[0] or [1] */
volatile int index = 0;                                 /* index for buffer */
int alt_buffer;                             /* switch for buffer [0] or [1] */
char snd_msg[30];                                           /* send message */
extern volatile pid_t ili_24b_irq_proxy;                      /* from int.c */
extern volatile int transfer_no;                              /* from int.c */
extern volatile pid_t timer_proxy;                            /* from int.c */
extern volatile int ms_counter;                               /* from int.c */

int value;                        /* data sample value converted to integer */
int n_ili_cards = 0;                      /* number of cards present in ILI */
int n_ili_channels = 0;            /* number of ILI channels (inc. dummies) */
int message_size;              /* size of data expected from ILI per second */
int transfers_per_sec;     /* no of interrupts expected from ILI per second */
int high_rc_flag = 0;                   /* greatest value found for rc_flag */



/****************************************************************************/
/*    FUNCTION:   start_ili_24b                                             */
/*                                                                          */
/*  start communications with the ILI unit                                  */
/****************************************************************************/
int start_ili_24b (void)
{


    atexit (stop_ili_24b);

    /* create a proxy for the IRQ handler to use */
    ili_24b_irq_proxy = qnx_proxy_attach ((pid_t) 0, "", 0, -1);
    if (ili_24b_irq_proxy == -1)
    {
      ili_24b_irq_proxy = (pid_t) 0;
      return FALSE;
    }

    /* set up the ILI IRQ handler */
    irq_handler_id = qnx_hint_attach (ILI_24B_IRQ, &ili_24b_irq_handler,
									FP_SEG (buffer));
    if (irq_handler_id == -1) return FALSE;


	/* attach proxy for timeout interrupt handler */
	timer_proxy = qnx_proxy_attach ((pid_t)0, "timeout", 8, -1);
	if (timer_proxy == -1)
	{
		timer_proxy = (pid_t) 0;
		return FALSE;
	}

	/* attach the timeout interrupt handler */
	timer_irq_handler_id = qnx_hint_attach (0, &timeout_handler, FP_SEG (buffer));
	if (timer_irq_handler_id == -1) return FALSE;

	system ("ticksize 1");     /* for the timeout interrupt */


    /* set up 8255 and enable IRQs - 8255 set up is as follows:
     *   Group A: Mode 0, A = Input, C (Upper) = Output
     *   Group B: Mode 0, B = Input, C (Lower) = Output
     * The IRQ enable for the PIO-12 card is connected to Port C, Bit 0
     * The ILI XACK (acknowledge) line is connected to Port C, Bit 1
     * The ILI PDMA line is connected to Port C, Bit 2
     * The 24-bit ILI data is connected to Port A (lower word) and B */
    outp (P8255_CTRL_PORT, 0x92);
    outp (P8255_PORT_C, 0x02);

    /* initialise transfer control protocol for interrupt handler */
    transfer_no = 0;

	/* now work out the correct transfer size */
    /* NB this depends on the seislog parameter file --  the check function
       must check that the no of ILI cards present matches what is expected */
    message_size = HEADER_SIZE + (n_ili_cards * DATA_SIZE);
    transfers_per_sec = message_size / 2;         /* 2 bytes per interrupt */
    return TRUE;

}
/****************************************************************************/
/*   end of function start_ili_24b                                          */
/****************************************************************************/


/****************************************************************************/
/*    FUNCTION:   stop_ili_24b                                              */
/*                                                                          */
/*  neat and tidy shutdown of ILI communications                            */
/****************************************************************************/
void stop_ili_24b (void)

{

    /* have IRQs been started */
    if (irq_handler_id != -1)
    {
        /* yes - disable 8255 IRQs */
        outp (P8255_PORT_C, 0x01);

        /* detach the interrupt handlers */
        qnx_hint_detach (irq_handler_id);
        irq_handler_id = -1;
	    qnx_hint_detach (timer_irq_handler_id);
	    timer_irq_handler_id = -1;
    }

    /* if a proxy was created the destroy it */
    if (ili_24b_irq_proxy != 0)
    {
        qnx_proxy_detach (ili_24b_irq_proxy);
        ili_24b_irq_proxy = (pid_t) 0;
    }
    if (timer_proxy != 0)
    {
	    qnx_proxy_detach (timer_proxy);
	    timer_proxy = (pid_t) 0;
    }

}
/****************************************************************************/
/*   end of function stop_ili_24b                                           */
/****************************************************************************/



/****************************************************************************/
/*    FUNCTION:   main                                                      */
/*                                                                          */
/****************************************************************************/
main()
{
  int           ili_24bit_id;            /* Process id for ILI_24BIT        */
  int           buf_pip_id;              /* Process id for BUF_PIP          */
  char          rep_msg[10];             /* reply message                   */
  char          rec_msg[10];             /* receive message                 */
  int           no_of_samples;           /* no of samples expected per sec  */
  int           sample_no;    /* to get order of samples within data stream */
  unsigned int  cop;                     /* for buf_pip name locate         */
  int           rc_flag;          /* ili channel number from parameter file */
  int fd;                              /* file descriptor for shared memory */
  int n, n1, n2, a1, a2;                             /* loop counting dudes */
  union Join hdr_length, data_length, numchan, data1, testpat1, testpat2,
             packcount, data2;            /* structures for data conversion */
  int count;                                  /* another loop counting dude */
  unsigned short int tempbyte1, tempbyte2; /* return value for convert_data */
  char *proxy_msg = "       ";                /* message from timeout proxy */
  int           sync_return_value;       /* return value from sync function */
  int           channel_errors_return_value;
                         /* return value from check_channel_errors function */
  union Join packcount_valid, timecorr, ppscount, err_flag;
                                             /* for debugging time stamping */
  unsigned long int last_packcount;            /* for packet validity check */
  char last_timestring[30];   /* for time validity on unmarked packet check */
  FILE *fp;                                /* for error logging if in debug */




setprio(getpid(),12);                  /* set the priority of this process  */

/*--------------------------------------------------------------------------*/
/* attach this process                                                      */
/*--------------------------------------------------------------------------*/
ili_24bit_id = qnx_name_attach(0,"ili_24bit");
if(ili_24bit_id == -1)             
{
    printf("ILI_24BIT attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd = shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd == -1) 
{
    fprintf(stderr,"ILI_24BIT open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
if (ltrunc(fd,sizeof(smod),SEEK_SET) == -1)
{
/*	fprintf (stderr, "ILI_24BIT ltrunc failed:%s\n",strerror(errno));
	exit(1);*/
}
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr = mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"ILI_24BIT mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id = qnx_name_locate(0,"buf_pip",0,&cop);
if (buf_pip_id == -1)
{
	fprintf (stderr, "ILI_24BIT: cannot locate buf_pip, is SEISLOG running?\n");
	exit(0);
}

/*--------------------------------------------------------------------------*/
/* build mapping table for ILI channels -> SEISLOG channels                 */
/*--------------------------------------------------------------------------*/
for (n=0; n < MAX_ILI_CHAN; n++)
{
    ilichan[n] = -1;
    sample_order[n] = -1;
}
for (a1 = 0; a1 < 16; a1++)
    for (a2 = 0; a2 < 3; a2++)
        transform_order[a1][a2] = -1;

for (n=0; n<ptr->active; n++)
{
    for (count=0; count<ptr->nchan; count++)
    {
        rc_flag = ptr->store[n][count];
        if (rc_flag > high_rc_flag)   high_rc_flag = rc_flag;   /* find max */
        ilichan[rc_flag - 1] = count + 1;   /* -1 and +1 since array from 0 */
    }
}


/* now find no of channels in ILI so we know what message size to expect */
n_ili_cards = high_rc_flag / 3;
if ((high_rc_flag % 3) > 0)   n_ili_cards++;  /* this gives no of cards */
n_ili_channels = n_ili_cards * 3;             /* all 3 component */

reorder_ilichannels();


//printf ("no of seislog channels = %d\n", ptr->nchan);
//printf ("no of ili channels = %d\n", n_ili_channels);
//printf ("*** sample_order: ");
//for (n=0; n<MAX_ILI_CHAN; n++) printf ("%d ",sample_order[n]);
//printf ("\n");


ptr->index = 0;                            /* initialise ringbuffer pointer */
ptr->alive[1] = 1;					                       /* say I'm alive */



/*--------------------------------------------------------------------------*/
/* start communicating with the ILI                                         */
/*--------------------------------------------------------------------------*/
if (! start_ili_24b ())
{
    fprintf (stderr, "ILI_SEIS: can't start comms for the ILI - are you running as root?\n");
    exit (1);
}
fprintf (stdout, "UNIT 99: 24 BIT ILI\n");
fprintf (stderr, "ILI_SEIS: synchronising with data stream ...   ");

if ((sync_return_value = sync(&last_packcount)) != 0)      /* get in sync with data stream */
{
    fprintf (stderr, "\nILI_SEIS: bad value from sync function\n");
    system ("home/seislog/stp");
    exit (1);
}                                  
fprintf (stderr, "done\n");

fprintf (stderr, "ILI_SEIS: checking ILI channels ...    ");
if ((channel_errors_return_value = check_channel_errors()) != 0)
                  /* check for discrepancies between ILI channels fitted and
                                            what seislog thinks are present */
{
    fprintf (stderr, "\nILI_SEIS: bad value from check_channel_errors function\n");
    system ("home/seislog/stp");
    exit (1);
}
fprintf (stderr, "done\n");


//fprintf (stdout, "ILI_SEIS: 24 bit driver started, channels %d data size %d\n", n_ili_channels, message_size);
Send (buf_pip_id, &snd_msg, &rec_msg, 1, 1);  /* send to buf_pip to give id */


/*--------------------------------------------------------------------------*/
/* main loop starts here                                                    */
/*--------------------------------------------------------------------------*/

for (;;)
{
    ptr->alive[1] = 1;                           /* say I'm alive */

 	Receive (0, (char *)proxy_msg, 8);       /* receive from ILI or timer */
	if (!strcmp (proxy_msg, "timeout"))      /* timeout */
	{
		fprintf (stderr, "ILI DRIVER ERROR: no data received from ILI\n");
		fprintf (stderr, "check ILI->PC connection\n");
		strset (proxy_msg, ' ');

	}
	else  ms_counter = 0;                  /* reset millisecond counter */

    alt_buffer=(current_buffer+1)%2;       /* switch buffer for reading */


	/* work out some stuff from the header */

/*	    for (count=0; count<4; count++)
		  hdr_length.byte[count] = buffer[alt_buffer][count+4];
	    for (count=0; count<4; count++)
	 	  data_length.byte[count] = buffer[alt_buffer][count+260];
	    for (count=0; count<2; count++)
		  numchan.byte[count] = buffer[alt_buffer][count+58];
		testpat1.byte[0] = buffer[alt_buffer][34];
		testpat1.byte[1] = buffer[alt_buffer][35];
		testpat2.byte[0] = buffer[alt_buffer][40];
		testpat2.byte[1] = buffer[alt_buffer][41];
*/		for (count=0; count<4; count++)
			packcount.byte[count] = buffer[alt_buffer][count+64];

/*		for (count=0; count<4; count++)
			data1.byte[count] = buffer[alt_buffer][count+264];
		for (count=0; count<4; count++)
			data2.byte[count] = buffer[alt_buffer][count+268];
*/
        /* these added for debugging time stamping 13/11/97 */
		for (count=0; count<4; count++)
			packcount_valid.byte[count] = buffer[alt_buffer][count+68];
		for (count=0; count<2; count++)
			timecorr.byte[count] = buffer[alt_buffer][count+76];
		for (count=0; count<2; count++)
			ppscount.byte[count] = buffer[alt_buffer][count+78];
		for (count=0; count<2; count++)
			err_flag.byte[count] = buffer[alt_buffer][count+82];
        


/*    printf ("%.3s %ld %ld %.4X %.4X %d %c %ld    %ld %d\n", buffer[alt_buffer],
								 hdr_length.word_32b,
								 data_length.word_32b,
								 testpat1.word_16b,
								 testpat2.word_16b,
								 numchan.word_16b,
								 buffer[alt_buffer][42],
								 packcount.word_32b,
								 value, tempbyte1);
  */                               

/***   For debugging purposes, uncommment this block  DJS 10/02/98
printf ("Packet: %ld, last valid %ld    %d %d %d\n", packcount.word_32b,
								      packcount_valid.word_32b,
                                      timecorr.word_16b,
                                      ppscount.word_16b,
                                      err_flag.word_16b);
printf ("Timestring: %.128s\n", &buffer[alt_buffer][128]);
fp = fopen ("mess_log", "a");
fprintf (fp,"Packet: %ld, last valid %ld    %d %d %d\n", packcount.word_32b,
								      packcount_valid.word_32b,
                                      timecorr.word_16b,
                                      ppscount.word_16b,
                                      err_flag.word_16b);
fprintf (fp,"Timestring: %.128s\n", &buffer[alt_buffer][128]);
if (packcount.word_32b == 0) fprintf (fp, "ignore this packet\n");
fclose (fp);
***/


    /* Workaround for error where an extra iteration of the loop is
       performed due to buffer overrun - we do nothing and go to the
       end of the loop.   DJS 03/02/98 */
    if (packcount.word_32b == 0)
        goto ignored;

        

 	/* store data in buffer */

	for (n1 = 0; n1 < 75; n1++)
	{
     for (n2 = 0; n2 < n_ili_channels; n2++)
        {

         if (sample_order[n2] > 0)
            {
             convert_data (&data1, (n1 * n_ili_channels + n2));
             ptr->ibuf[ptr->index][(n1 * ptr->nchan) + (sample_order[n2] -1) + HEADER] = value;
          }
     }
	}


//convert2();   /* dig_syn type function - development only */


    /* added 13/11/97 DJS */
    if ((packcount.word_32b != packcount_valid.word_32b)) /* header time unreliable */
    {

/***  For debugging purposes, uncomment this block   DJS 10/02/98
fp = fopen ("mess_log", "a");
fprintf (fp, "calling increment time routine\n");
if (packcount.word_32b != packcount_valid.word_32b) fprintf (fp, "packet count indicates not valid\n");
fclose(fp);
***/

        /* add one second to last time string */
        increment_last_time();         
    }
    else
    {
        /* read the timestamp info from the ILI header */
        extract_timestring(); 
     }

    /* Workaround for error where packet is indicated valid but
       does NOT contain a valid timestring - if it's the same as
       last time's, increment it.   DJS 03/02/98 */
    if (!(strcmp (snd_msg, last_timestring)))
    {
        increment_last_time();

/***   For debugging purposes, uncomment this block   DJS 10/02/98
fp = fopen ("mess_log", "a");
fprintf (fp, "Packet indicated valid but incremented time\n");
fclose(fp);
***/

    }


/***   For debugging purposes, uncomment this block   DJS 10/02/98
fp = fopen ("mess_log", "a");
fprintf (fp, "%.30s\n\n", snd_msg);
fclose(fp);
***/

	Send (buf_pip_id, &snd_msg, &rec_msg, 30, 1);   /* and send to buf_pipe */
    last_packcount = packcount.word_32b;
    strcpy (last_timestring, snd_msg);

ignored:    ;   /* label for packet error */

}   /*** end of forever loop  ***/

}
/****************************************************************************/
/*   end of main function                                                   */
/****************************************************************************/

/****************************************************************************/
/*    FUNCTION:   sync                                                      */
/*                                                                          */
/*  synchronise with data stream from ILI                                   */
/*    (otherwise the first data we get will be rubbish)                     */
/****************************************************************************/

int sync(int *last_packcount)
{
 int in_sync = FALSE,
     ili_test_pattern = FALSE,
     dat_test_pattern = FALSE,
	 hdr_length_test = FALSE;
 int count;
 union Join hdr_length, packcount;


  do
  {
      Receive (ili_24b_irq_proxy, (char *) 0, 0);  /* receive data from ILI */

      if (!strncmp (buffer[alt_buffer], "ILI", 3))	ili_test_pattern = TRUE;
      if (!strncmp (&buffer[alt_buffer][256], "DAT", 3))	dat_test_pattern = TRUE;
      for (count=0; count<4; count++)
		  hdr_length.byte[count] = buffer[alt_buffer][count+4];
	  if (hdr_length.word_32b == 248)	hdr_length_test = TRUE;

      if ((ili_test_pattern == TRUE) && 
          (dat_test_pattern == TRUE) && 
          (hdr_length_test == TRUE)) 
		 in_sync = TRUE;
  }
  while (in_sync == FALSE);
  for (count=0; count<4; count++)
 	packcount.byte[count] = buffer[alt_buffer][count+64];
  *last_packcount = packcount.word_32b -1;

  return (0);
}
/****************************************************************************/
/*   end of function sync                                                   */
/****************************************************************************/



/****************************************************************************/
/*    FUNCTION:   check_channel_errors                                      */
/*                                                                          */
/*  check that the number of ILI channels fitted matches what SEISLOG       */
/*    expects from the parameter file                                       */
/****************************************************************************/

int check_channel_errors (void)
{
 int count;
 char channels_present[16];
 int total_channels;
 int nonsense_number;



    do
    {
      Receive (ili_24b_irq_proxy, (char *) 0, 0);  /* receive data from ILI */

      total_channels = 0;
      nonsense_number = FALSE;

      for (count=0; count<16; count++)
      {
          channels_present[count] = buffer[alt_buffer][42 + count];
          switch (channels_present[count])
          {
              case '0':
                  break;
              case '1':
                  printf ("ILI_SEIS:  single component cards are not compatible with SEISLOG!\n");
                  printf ("Remove ILI card no. %d\n", (count+1));
                  printf ("ILI driver stopped\n");
                  system ("/home/seislog/stp");
                  exit (0); 
                  break;
              case '3':
                  total_channels++;
                  break;
              default:
                  nonsense_number = TRUE;
                  break;
          }
      }
    } while (nonsense_number);


    if (total_channels > n_ili_cards)   /* more ILI cards than SEISLOG knows about */
    {
        message_size = HEADER_SIZE + (total_channels * DATA_SIZE);
        transfers_per_sec = message_size / 2;         /* 2 bytes per interrupt */
        n_ili_cards = total_channels;
        n_ili_channels = total_channels * 3;
        reorder_ilichannels();
    }

    if (n_ili_cards > total_channels)
    {
        printf ("ILI_SEIS: More SEISLOG channels have been defined than ILI channels available\n");
        printf ("Adjust parameter file before restarting SEISLOG\n");
        system ("/home/seislog/stp");
        exit (0);
    }
    return (0);

}
/****************************************************************************/
/*   end of function check_channel_errors                                   */
/****************************************************************************/



/***  -- this function for development only --  ***/
/*** function adapted from dig_syn ***/
void convert2 (void)
{
 int lg;
 int indx=0;
 float s;

    for (lg=0; lg<100; lg++)
    	{
			s = sin (3.1415 * (ptr->srate) * (float)lg) * 100.0;
			ptr->ibuf[ptr->index][indx+HEADER]= (int)s + 100;
        	indx=indx+1;
	  	}
}



/****************************************************************************/
/*    FUNCTION:   convert_data                                              */
/*                                                                          */
/*  convert buffered data samples to integer format                         */
/*    this function returns the status byte from the data sample            */
/*    the value of the sample is written into the global "value"            */
/*    the data_word'th sample from the driver's internal buffer is read     */
/****************************************************************************/

unsigned short int convert_data (union Join* join, int data_word)
{
  unsigned short int lowbyte;
  unsigned char lo,me,hi;
  unsigned int val;
  int va;
	lowbyte = buffer[alt_buffer][data_word*4 + 264];
    /* bottom byte is status fluff so dump it by right shifting */

    val=0;
    lo=buffer[alt_buffer][data_word*4 + 265];
    me=buffer[alt_buffer][data_word*4 + 266];
    hi=buffer[alt_buffer][data_word*4 + 267];
    val=hi;
    val=val<<8 | me;
    val=val<<8 | lo;
    val=val<<8;
    va=(int)val;
    value=va/256;

	return lowbyte;
}
/****************************************************************************/
/*   end of function convert_data                                           */
/****************************************************************************/



/****************************************************************************/
/*    FUNCTION:   reorder_ilichannels                                       */
/*                                                                          */
/*  change the order of the ILI channels for storage in buf_pip             */
/*    we call the ILI channels 1 2 3 for card 1, 4 5 6 for card 2, etc      */
/*    the ILI actually sends data samples in the order 1 4 7 2 5 8 3 6 9    */
/*    this function builds a new array based on this order                  */
/****************************************************************************/

void reorder_ilichannels(void)
{
  int array_pos;
  int n, count;

    for (n = 0; n < 16; n++)
    {
        transform_order[n][0] = ilichan[n*3];
        transform_order[n][1] = ilichan[n*3 + 1];
        transform_order[n][2] = ilichan[n*3 + 2];
    }

    /* NB The ilichan array expects to receive samples in the order
            card1 channel 1 channel 2 channel 3 card2 channel 1 etc
        but in fact the ILI sends
            card1 channel 1 card2 channel 1 card3 channel 1 card1 channel 2
        so now we have to produce a new array using this order
        which we will apply when storing the samples
    */

    array_pos = 0;
    for (n=0; n < 3; n++)
    {
        for (count = 0; count < n_ili_cards; count++)
        {
            sample_order[array_pos] = transform_order[count][n];
            array_pos++;
        }
    }
} 
/****************************************************************************/
/*   end of function reorder_ilichannels                                    */
/****************************************************************************/


/****************************************************************************/
/*    FUNCTION:   extract_timestring                                        */
/*                                                                          */
/*  extracts the timestring information from the ILI header                 */
/*    and builds it into the snd_msg string which will be sent to buf_pipe  */
/****************************************************************************/

void extract_timestring (void)
{
  char *string_walker;
  int count;


    for (count=0; count < 30; count++)
        snd_msg[count] = ' ';                       /* initialize string */

    string_walker = &buffer[alt_buffer][128];
    while (*string_walker != ',') string_walker++;
    string_walker++;                                    /* eat the comma */
    snd_msg[0] = *string_walker;    string_walker++;         /* hours    */
    snd_msg[1] = *string_walker;    string_walker++;
    snd_msg[3] = *string_walker;    string_walker++;         /* minutes  */
    snd_msg[4] = *string_walker;    string_walker++;
    snd_msg[6] = *string_walker;    string_walker++;         /* seconds  */ 
    snd_msg[7] = *string_walker;    string_walker++;
    if (*string_walker != ',')                  /* milliseconds present */
    {
        snd_msg[8] = '.';                string_walker++;
        snd_msg[9] = *string_walker;     string_walker++;
        snd_msg[10] = *string_walker;    string_walker++;
        snd_msg[11] = *string_walker;    string_walker++;
    }
    else                                       /* make milliseconds zero */
    {
        snd_msg[8] = '.';
        snd_msg[9] = '0';
        snd_msg[10] = '0';
        snd_msg[11] = '0';
    }
    string_walker++;                                   /* eat the comma */
    for (count=0; count < 7; count++)
    {
        while (*string_walker != ',') string_walker++;
        string_walker++;
    }
    snd_msg[13] = *string_walker;    string_walker++;          /* day   */
    snd_msg[14] = *string_walker;    string_walker++;
    snd_msg[16] = *string_walker;    string_walker++;          /* month */
    snd_msg[17] = *string_walker;    string_walker++;
    snd_msg[19] = *string_walker;    string_walker++;          /* year  */
    snd_msg[20] = *string_walker;    string_walker++;

 
}

/****************************************************************************/
/*   end of function extract_timestring                                     */
/****************************************************************************/



/****************************************************************************/
/*    FUNCTION:   increment_last_time                                       */
/*                                                                          */
/*  if the timestring information from the ILI header is not valid,         */
/*    add one second to the snd_msg string which will be sent to buf_pipe   */
/****************************************************************************/

void increment_last_time (void)
{
  char hourstr[3],minstr[3],secstr[3],daystr[3],monstr[3],yearstr[3];
  int hour,min,sec,day,mon,year,doy;

    secstr[0] = snd_msg[6];
    secstr[1] = snd_msg[7];
    secstr[2] = '\0';
    sec = atoi(secstr);
    sec++;
    if (sec == 60)                              /* new minute */
    {
        sec = 0;
        minstr[0] = snd_msg[3];
        minstr[1] = snd_msg[4];
        minstr[2] = '\0';
        min = atoi(minstr);
        min++;
        if (min == 60)                         /* new hour */
        {
            min = 0;
            hourstr[0] = snd_msg[0];
            hourstr[1] = snd_msg[1];
            hourstr[2] = '\0';
            hour = atoi(hourstr);
            hour++;
            if (hour == 24)                   /* new day */
            {
                hour = 0;
                daystr[0] = snd_msg[13];
                daystr[1] = snd_msg[14];
                daystr[2] = '\0';
                monstr[0] = snd_msg[16];
                monstr[1] = snd_msg[17];
                monstr[2] = '\0';
                yearstr[0] = snd_msg[19];
                yearstr[1] = snd_msg[20];
                yearstr[2] = '\0';
                day = atoi(daystr);
                mon = atoi(monstr);
                year = atoi(yearstr);
                doy = date_to_day_of_year (day, mon, year);
                doy++;
                if (((doy == 366) && (!is_leap_year(year))) ||
                    ((doy == 367) && (is_leap_year(year))))
                {
                    day = 1;
                    mon = 1;
                    year++;
                }
                else day_of_year_to_date (doy, year, &day, &mon);
                itoa (day, daystr, 10);
                itoa (mon, monstr, 10);
                itoa (year, yearstr, 10);
                snd_msg[13] = daystr[0];
                snd_msg[14] = daystr[1];
                snd_msg[16] = monstr[0];
                snd_msg[17] = monstr[1];
                snd_msg[19] = yearstr[0];
                snd_msg[20] = yearstr[1];
            }
            itoa (hour, hourstr, 10);
            if (hour < 10)
            {
                snd_msg[0] = 0;
                snd_msg[1] = hourstr[0];
            }
            else
            {
                snd_msg[0] = hourstr[0];
                snd_msg[1] = hourstr[1];
            }
        }
        itoa (min, minstr, 10);
        if (min < 10)
        {
            snd_msg[3] = '0';
            snd_msg[4] = minstr[0];
        }
        else
        {
            snd_msg[3] = minstr[0];
            snd_msg[4] = minstr[1];
        }
    }
    itoa (sec, secstr, 10);
    if (sec < 10)
    {
        snd_msg[6] = '0';
        snd_msg[7] = secstr[0];
    }
    else
    {
        snd_msg[6] = secstr[0];
        snd_msg[7] = secstr[1];
    }

}

/****************************************************************************/
/*   end of function increment_last_time                                    */
/****************************************************************************/
