/*************************************************************************

TIME PROCEDURES

TIMSEC -  Converts time in year, month, day, hour, minutes and seconds
          to double precision seconds after 1/1/1900
SECTIM -  Converts double precision seconds after 1/1/1900 to year,
          day-of-year and date
DTE    -  Converts day-of-year to date
TIMDIF -  Calculates time-difference in double precision secondes of two
          input dates (year,month,day,hour,minute,second)
TIMADD -  Adds double precision seconds to date, and returns the new date

Original Fortran routines by Leif Kvamme 9-4-87

Translated into C by Jane Exton 21/10/92

******************************************************************************/

/* structure definition of date */
#define START 1900
#define TRUE 1
#define FALSE 0

#define SECPERHOUR 3600.0
#define SECPERDAY 86400.0
#define SECPERYEAR 31536000.0

struct date {
             int year;
             int month;
             int day;
             int hour;
             int min;
             int doy;            /* day of year */
             float secs;         /* seconds     */
             double abs_secs;     /* seconds from 1/1/1900 */
            };

void sectim();
void timsec();
void dte();
double timdif();
void timadd();



/* In all of the following procedures, a structure of type DATE can be
   passed, and the relevant fields will be filled in */


/*****************************************************************************
TIMSEC
*****************************************************************************/

      void timsec(dat)
           struct date* dat;

      {
        int no_of_leap_years;   /* num of leap years since 1900 */
        int leap_year=FALSE;          /* flag for leap year */
        int extra_day=0;        /* to compensate for leap year */
        int days_in_current_year=0;

        if (dat->year >= START) dat->year -= START;
        if ((dat->year%4)==0){
          leap_year = TRUE;
          extra_day = 1;
          }
        no_of_leap_years = dat->year/4 - extra_day; /* division is integer
                                                       here */
        dat->abs_secs = (float)(no_of_leap_years*366) +
                        (float)((dat->year - no_of_leap_years)*365);
                             /* days to beginning of year */

        dat->abs_secs *= SECPERDAY; /* seconds in a day */



        switch (dat->month){
         case 1:
           days_in_current_year = dat->day;
           break;
         case 2:
           days_in_current_year = dat->day + 31;
           break;
         case 3:
           days_in_current_year = dat->day + extra_day + 59;
           break;
         case 4:
           days_in_current_year = dat->day + extra_day + 90;
           break;
         case 5:
           days_in_current_year = dat->day + extra_day + 120;
           break;
         case 6:
           days_in_current_year = dat->day + extra_day + 151;
           break;
         case 7:
           days_in_current_year = dat->day + extra_day + 181;
           break;
         case 8:
           days_in_current_year = dat->day + extra_day + 212;
           break;
         case 9:
           days_in_current_year = dat->day + extra_day + 243;
           break;
         case 10:
           days_in_current_year = dat->day + extra_day + 273;
           break;
         case 11:
           days_in_current_year = dat->day + extra_day + 304;
           break;
         case 12:
           days_in_current_year = dat->day + extra_day + 334;
           break;
         }

         dat->abs_secs += (float)(days_in_current_year*SECPERDAY+
                           dat->hour*SECPERHOUR+dat->min*60.0);

         dat->abs_secs +=(double)dat->secs;

         return;

}  /* end of TIMSEC */


/*****************************************************************************
SECTIM - this routine calculates only from 1/1/1901 to 31/12/2099
******************************************************************************/

              void sectim(dat)
                   struct date* dat;


   {
     double sec=0;
     int i;  /* counter */
     float no_of_leap_years = 0;
     float extra_days=0;
     float leap_year =0;


/* Find year */
      for(i=1;i<=200;i++){
         extra_days = 0;
         if(i%4 == 0) extra_days = 1;
         sec += SECPERYEAR + extra_days*SECPERDAY;
         if(sec > dat->abs_secs) break;  /* year found */
         leap_year = extra_days;  /* leap_year is set if the year is a leap
                                     year */
         }

   dat->year = i-1;

/* Find doy and date */
                            /* reset sec to beginning of year */
      sec = dat->abs_secs - (sec - SECPERYEAR - extra_days*SECPERDAY);
      for(i=1;i<366;i++){
         if((sec-=SECPERDAY)<0) break;
         }

      i--;   /* set to doy */
      if(leap_year>0) i++;
      if (i==0) {
         dat->year--;
         i=365;
         if(dat->year%4==0) i=366;
         }
      dat->doy = i;

      dte(dat);    /* procedure finds the date from the doy */

/* Find hour */
      sec += SECPERDAY;
      for(i=1;i<=24;i++){
         sec -= SECPERHOUR;
         if(sec <0){
           break;
           }
         }

      dat->hour = i-1;

/* Find minutes and remaining seconds */

      sec += SECPERHOUR;
      for(i=1;i<=60;i++){
        sec -= 60.0;
        if( sec <0) break;
        }

      dat->min = i-1;
      dat->secs = sec+60;

      return;

}


/****************************************************************************
DTE  converts from day of year to date
****************************************************************************/

            void dte(dat)
               struct date* dat;

{
  int months[12];
  int j,m=0,n;   /* counters */

  months[0]=months[2]=months[4]=months[6]=months[7]=months[9]=months[11]
                                                              =31;
  months[1]=28;
  months[3]=months[5]=months[8]=months[10]=30;




  if(dat->year%4==0) months[1]=29;
  for(j=0;j<12;j++){
     m += *(months+j);
     n = dat->doy - m;
     if(n<0){
       dat->month = (j+1);
       dat->day = n + months[j];
       break;
       }
   } /* end of for loop */

   return;

}




/*****************************************************************************
TIMDIF  - time difference in seconds is calculated from two dates
          answer will be returned as the difference of the second date
          from the first
*****************************************************************************/

          double timdif(dat1, dat2)
                 struct date *dat1, *dat2;

 {
   /* convert both to absolute secs */
          timsec(dat1);
          timsec(dat2);

   /* calculate difference */
          return(dat1->abs_secs - dat2->abs_secs);

  } /* end of timdif */


/*****************************************************************************
TIMADD - adds time in seconds to a date.

*****************************************************************************/

             void timadd(dat1,dat2,sec)
                    struct date *dat1, *dat2;
                    double sec;

             {
               timsec(dat1);  /* absolute seconds returned in dat->abs_secs*/

               dat2->abs_secs = dat1->abs_secs+sec;

               sectim(dat2);
             } /* end of timadd */


